import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System import Math, Action, Func
from System.Threading import Thread
from System.Text import Encoding
from System.Collections.Generic import List
from System.Windows import Visibility
from System.Windows.Controls import StackPanel
from System.Windows.Input import Keyboard, AccessKeyManager
from System import Type, Tuple, Array, Double
from System.IO import StreamReader, MemoryStream, File
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle
from Trimble.Vce.SurveyCAD import ProjectLineStyle
from Trimble.Vce.ForeignCad import Arc as ArcObject # we are also using Arc from geometry
from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings
from Trimble.Vce.Geometry import Point3D, Plane3D, Arc, Matrix4D, Vector3D
from Trimble.Vce.Interfaces import Client
clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import ViewHelper
clr.AddReference("Trimble.Vce.UI.Controls")
try:
    from Trimble.Sdk.UI import InputMethod, MousePosition, CursorStyle    
except:
    from Trimble.Sdk.Interfaces.UI import InputMethod, MousePosition, CursorStyle

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "3DPtsToArc"
    cmdData.CommandName = "3DPtsToArc"
    cmdData.Caption = "_3D Pts To Arc"
    cmdData.UIForm = "PtsToArc"
    cmdData.HelpFile = "Macros.chm"
    cmdData.HelpTopic = "22242"
    try:
        cmdData.Version = 1.52
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2018/10/22/tml-3d-points-to-arc"
    except:
        pass

    try:
        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "CAD"
        cmdData.ShortCaption = "Pts To Arc"
        cmdData.ToolTipTitle = "Pts To Arc"
        cmdData.ToolTipTextFormatted = "Create an arc on a plane defined by three 3-D points."

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

class PtsToArc(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        local = Thread.CurrentThread.CurrentUICulture.Name.Substring(0,2)
        file = macroFileFolder + r"\3DPtsToArc." + local + ".xaml"
        if File.Exists(file):
            with StreamReader(file) as s:
                wpf.LoadComponent(self, s)
        else:
            with StreamReader(macroFileFolder + r"\3DPtsToArc.xaml") as s:
                wpf.LoadComponent(self, s)

        self.currentProject = currentProject

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = self.TryFindResource("Apply")
        buttons[1].Content = self.TryFindResource("Close")
        self.Caption = cmd.Command.Caption

        self.coordCtl1.ValueChanged += self.Coord1Changed
        self.coordCtl2.ValueChanged += self.Coord2Changed
        self.coordCtl3.ValueChanged += self.Coord3Changed
        #set default name/layer
        settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
        self.name.Text = settings.CreateLinestringSettings.Name
        self.Layers.SelectedSerialNumber = settings.CreateLinestringSettings.Layer

        # We want to start cmd with focus on coordinate control.  It's a bit
        # tricky to set focus in the OnLoad method.  The following 3 lines do
        # that.
        def SetFocusToControl():
            Keyboard.Focus(self.coordCtl1)
        Dispatcher.BeginInvoke(Dispatcher.CurrentDispatcher, Action(SetFocusToControl))

    def Coord1Changed(self, ctrl, e):
        self.coordCtl2.CursorStyle = CursorStyle.CrossHair | CursorStyle.RubberLine
        if self.coordCtl1.ResultCoordinateSystem:
            self.coordCtl2.AnchorPoint = MousePosition(self.coordCtl1.ClickWindow, self.coordCtl1.Coordinate, self.coordCtl1.ResultCoordinateSystem)
        else:
            self.coordCtl2.AnchorPoint = None

    def Coord2Changed(self, ctrl, e):
        self.coordCtl3.CursorStyle = CursorStyle.CrossHair | CursorStyle.RubberLine
        if self.coordCtl2.ResultCoordinateSystem:
            self.coordCtl3.AnchorPoint = MousePosition(self.coordCtl2.ClickWindow, self.coordCtl2.Coordinate, self.coordCtl2.ResultCoordinateSystem)
        else:
            self.coordCtl3.AnchorPoint = None

    def Coord3Changed(self, ctrl, e):
        # set keyboard focus if change was due to mouse pick
        if e.Cause == InputMethod.Mouse:     
            self.OkClicked(None, None)

    def CancelClicked(self, thisCmd, args):
        #save default name/layer
        settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
        settings.CreateLinestringSettings.Name = self.name.Text
        settings.CreateLinestringSettings.Layer = self.Layers.SelectedSerialNumber
        thisCmd.CloseUICommand()

    def OkClicked(self, thisCmd, e):
        ctrl = self.coordCtl1
        p1 = self.coordCtl1.Coordinate
        if not p1.Is3D :
            self.coordCtl1.StatusMessage = self.TryFindResource("NoCoord")
            return False
        self.coordCtl1.StatusMessage = ""

        p2 = self.coordCtl2.Coordinate
        if not p2.Is3D :
            self.coordCtl2.StatusMessage = self.TryFindResource("NoCoord")
            return False
        self.coordCtl2.StatusMessage = ""

        p3 = self.coordCtl3.Coordinate
        if not p3.Is3D :
            self.coordCtl3.StatusMessage = self.TryFindResource("NoCoord")
            return False
        self.coordCtl3.StatusMessage = ""

        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)
        try:
            # unroll any changes if something goes wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                container = ViewHelper.GetContainerForView(ctrl.ClickWindow)
                
                p = Plane3D(p1,p2,p3)
                p = p[0] # the plane is returned as first element
                if not p.IsValid:
                    return
                v = p.normal
                if v.Length < Point3D.Tolerance:
                    return
                v.Normalize()
                origin = Point3D((p1.X + p2.X + p3.X) / 3.0,
                        (p1.Y + p2.Y + p3.Y) / 3.0,
                        (p1.Z + p2.Z + p3.Z) / 3.0)
        
                # we have to do some tricky stuff to compute transform
                # get horz vector to check special case
                newXAxis = v.GetHorizontalOrthgonalVector(1.0 / 64)
                matrix = Matrix4D(origin,  newXAxis, v)
                matrixWorldToProjection = Matrix4D.Inverse(matrix)

                # get points in projection
                pts = Matrix4D.TransformPoints(matrixWorldToProjection, [p1,p2,p3])
                r = Arc.GetThreePointArcProperties(pts[0], pts[1], pts[2])  
                a = container.Add(clr.GetClrType(ArcObject))
                  
                radPoint = r[3]
                radPoint = Point3D(radPoint.X, radPoint.Y, 0.0)
                radPointWorld = Matrix4D.TransformPoint(matrix, radPoint)
                a.CenterPoint = radPointWorld
                a.Radius = r[4]
                a.Normal = matrix.Row(2)

                if r[5]: # isDirectionToRight
                    angleStart = (pts[2] - radPoint).Angle
                    angleEnd = (pts[0] - radPoint).Angle
                else:
                    angleStart = (pts[0] - radPoint).Angle
                    angleEnd = (pts[2] - radPoint).Angle
                a.StartAngle = angleStart
                a.EndAngle = angleEnd

                #set name/layer each time to get changes
                a.Name = self.name.Text
                sn = self.Layers.SelectedSerialNumber
                if sn:
                    a.Layer = sn
                # you MUST commit the change or they will all be un-done
                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

        Keyboard.Focus(self.coordCtl1)
        return