﻿"""
Compute polyline in profile view showing elevation difference between selected line and corridor
"""
#region Reference stuff
import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System import Math, Action, Func, Type, UInt32
from System.Text import Encoding
from System.Collections.Generic import List
from System.Windows import Window, MessageBox, MessageBoxResult, MessageBoxButton, Visibility
from System.Windows.Controls import StackPanel, Button, CheckBox
from System.Windows.Input import Keyboard, AccessKeyManager, Key
from System import Type, Tuple, Array, Double, String
from System.IO import StreamReader, MemoryStream, Path, File
from System.Globalization import CultureInfo
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap, Color
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle
from Trimble.Vce.SurveyCAD import ProjectLineStyle

from Trimble.DiskIO import OptionsManager

from Trimble.Vce.Alignment import ProfileView
from Trimble.Vce.Alignment.Linestring import Linestring
from Trimble.Vce.Corridor import Corridor, LevelContainer, Level, AvgEndAreaEarthworkEngine, IManifestedTemplate, RoadCodeType

from Trimble.Vce.ForeignCad import TextEntity, MText, Text, PolyLine

from Trimble.Vce.Gem import Model3D
from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings
from Trimble.Vce.Geometry import Point3D, Plane3D, Arc, Matrix4D, Vector3D, Side
from Trimble.Vce.Geometry.PolySeg import PolySeg

from Trimble.Vce.Interfaces.SnapIn import ISnapIn, IPolyseg, IHaveUcs, IMemberManagement, IName
from Trimble.Vce.Interfaces.Point import IPoint, IPointLocation
from Trimble.Vce.Interfaces.Units import CoordinateOrder, LinearType, VerticalAngleType, AzimuthType, AngularType
from Trimble.Vce.Interfaces.Client import CommandGranularity
from Trimble.Vce.Interfaces import Client, ProgressBar
clr.AddReference("Trimble.Vce.UI.Controls")
from Trimble.Vce.Core import GlobalSelection
from Trimble.Vce.UI.Controls import VceMessageBox
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "ComputeElevDiffToCorridor"
    cmdData.CommandName = "ComputeElevDiffToCorridor"
    cmdData.Caption = "_ComputeElevDiffToCorridor"
    cmdData.UIForm = "ComputeElevDiffToCorridor"

    # if you are going to include a help file, define file and topic id here
    cmdData.HelpFile = "MyMacros.chm"
    try:
        cmdData.Version = 0.52
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r""
    except:
        pass   

    try:
        # if you want the macro included on ribbon, fill in the following 5
        # lines
        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "Corridor"
        cmdData.ShortCaption = "Elev diff to Corridor"
        cmdData.ToolTipTitle = "Elev diff to Corridor"
        cmdData.ToolTipTextFormatted = "Compute elevation difference between line and corridor."

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

# the name of this class must match name from cmdData.UIForm (above)
class ComputeElevDiffToCorridor(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\ComputeElevDiffToCorridor.xaml") as s:
            wpf.LoadComponent(self, s)

        self.macroName = "Compute Elev Diff To Corridor"
        self.tmlFileFolder = macroFileFolder  

        self.currentProject = currentProject
        self.polyType = clr.GetClrType(IPolyseg)
        self.alignmentType = clr.GetClrType(ProfileView)
        self.objs.IsEntityValidCallback = self.IsValid

        self.corridor.SetEntityType(clr.GetClrType(Corridor), currentProject)
        levelContainer = LevelContainer.ProvideLevelContainer(currentProject)
        defaultLevelId = levelContainer.DefaultSerialNumber
        self.materialLayer.SearchContainer = levelContainer.SerialNumber
        self.materialLayer.UseSelectionEngine = False
        self.materialLayer.SetEntityType(clr.GetClrType(Level), currentProject)
        levelName = OptionsManager.GetString("ComputeElevDiffToCorridor:levelName", "")
        # try to match name of level
        for l in levelContainer:
            if String.Compare(levelName, l.Name, True) == 0:
                self.materialLayer.SelectBySerialNumber(defaultLevelId)
                break
        else:
            self.materialLayer.SelectBySerialNumber(l.SerialNumber)
            
        self.verticalOffsetCtl.Distance = OptionsManager.GetDouble("ComputeElevDiffToCorridor:verticalOffsetCtl", Double.NaN)

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Compute"
        buttons[1].Content = "Close"
        self.Caption = cmd.Command.Caption

    def CancelClicked(self, sender, e):
        sender.CloseUICommand()

    def IsValid(self, serial):
        o = self.currentProject.Concordance.Lookup(serial)
        return self.IsValidObj(o)

    def IsValidObj(self, obj):
        if isinstance(obj, self.alignmentType):
            return False
        if isinstance(obj, self.polyType):
            return True
        return False

    def OkClicked(self, sender, e):
        cor = self.corridor.SelectedEntity()
        if not cor:
            self.corridor.StatusMessage = "Select corridor"
            Keyboard.Focus(self.corridor)
            return False
        self.corridor.StatusMessage = ""

        verticalOffset = self.verticalOffsetCtl.Distance
        if Double.IsNaN(verticalOffset):
            self.verticalOffsetCtl.StatusMessage = "Enter valid vertical offset"
            return False
        self.verticalOffsetCtl.StatusMessage = ""

        self.materialSerial = self.materialLayer.SelectedEntity().SerialNumber
        hal = cor.AlignmentSerialNumber
        hal = self.currentProject.Concordance.Lookup(hal)
        self.alignPolyseg = hal.ComputePolySeg().Clone()
        self.alignPolyseg.Extend(10000.0)
        with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
            for o in self.objs.SelectedMembers(self.currentProject):
                if not self.IsValidObj(o):
                    continue
                polySeg = o.ComputePolySeg()
                if not polySeg:
                    continue
                self.ComputeDiffereneLine(o, polySeg, hal, cor, verticalOffset)
            failGuard.Commit()
            
        OptionsManager.SetValue("ComputeElevDiffToCorridor:verticalOffsetCtl", self.verticalOffsetCtl.Distance)
        self.currentProject.TransactionManager.AddEndMark(CommandGranularity.Command)

        sender.CloseUICommand()
    
    def ComputeDiffereneLine(self, line, linePoly, alignment, corridor, verticalOffset):
        polyline = None
        for node in linePoly.Nodes():
            rtn = self.alignPolyseg.FindPointFromPoint(node.Point, None, None)
            if not rtn[0]:
                continue
            station = rtn[4]
            offset = rtn[5]
            manifestedTemplate = corridor.GetManifestedRawData(station).ManifestedTemplate
            segments = IManifestedTemplate.Segments.__get__(manifestedTemplate)
            if segments: 
                for segment in segments:
                    if not segment.IsMemberOfLevel(self.materialSerial):
                        continue
                    if segment.Node1.Offset < segment.Node2.Offset:
                        off1 = segment.Node1.Offset
                        off2 = segment.Node2.Offset
                        el1 = segment.Node1.Elevation
                        el2 = segment.Node2.Elevation
                    else:
                        off1 = segment.Node2.Offset
                        off2 = segment.Node1.Offset
                        el1 = segment.Node2.Elevation
                        el2 = segment.Node1.Elevation

                    # If our offset is between the segment points, use this segment
                    if offset >= off1 and offset <= off2:
                        percent = (offset - off1) / (off2 - off1)
                        deltaElev = el2 - el1
                        elevation = (deltaElev * percent) + el1
                        if not polyline:
                            polyline = alignment.Add(clr.GetClrType(PolyLine))
                            polyline.Layer = line.Layer
                            polyline.Name = IName.Name.__get__(line)
                        n = PolyLine.PolyLineNode()
                        n.X = station
                        n.Y = elevation - (node.Point.Z + verticalOffset)
                        polyline.AddNode(n)
