import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System import Uri, Object
from System.Reflection import BindingFlags
from System.Collections import ICollection
from System.Windows.Controls import StackPanel, Image
from System.Windows.Media.Imaging import BitmapImage, BitmapCacheOption
from System.Windows.Media  import Stretch 
from System.IO import StreamReader, Path, File, FileMode, MemoryStream, FileStream, FileAccess
from System.Windows.Input import Keyboard
from System import Type, Tuple, Array, Double
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle, BlockCollection, BlockView, \
        LayerGroupCollection, LayerGroup

from Trimble.Vce.Interfaces.Client import CommandGranularity, IDuplicator
from Trimble.Vce.Interfaces.SnapIn import IMemberManagement, IViewMember, ISpacialContainer, \
        IDuplicatableTargetContainer, IContainerInternals, IDuplicatable, ICopy, IRepresentGraphically
from Trimble.Vce.Interfaces.Core import IDuplicatorInternals
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents

try:
    from Trimble.Sdk.UI import UIEventArgs 
except:
     from Trimble.Sdk.Interfaces.UI import UIEventArgs

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "CopyLayerGroupMembers"
    cmdData.Caption = "_Copy Layer Group Members"
    cmdData.UIForm = "CopyLayerGroupMembers"
    try:
        cmdData.Version = 1.51
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2018/12/05/tml-copy-layer-group-members"
        cmdData.HelpFile = cmdData.MacroInfo
    except:
        pass

    try:
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

class CopyLayerGroupMembers(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\CopyLayerGroupMembers.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject
        self.macroFolder = macroFileFolder
        lgCollection = LayerGroupCollection.GetLayerGroupCollection(self.currentProject, False)
        if lgCollection:
            self.layerGroupsCtl.SearchContainer = lgCollection.SerialNumber
            self.layerGroupsCtl.UseSelectionEngine = False
            self.layerGroupsCtl.AllowNone = False
            # this allows both the default and user created regions to be
            # listed.
            self.layerGroupsCtl.SetEntityType(clr.GetClrType(LayerGroup), currentProject)

    def OnLoad(self, cmd, buttons, event):
        self.Caption = cmd.Command.Caption

    def layerGroupsCtl_ValueChanged(self, sender, e):
        lg = self.layerGroupsCtl.SelectedEntity()
        if lg:
            prefix = self.groupPrefix.Text
            lgn = lg.Name
            self.newGroupName.Text = self.UpdateName(prefix, lgn)

    def UpdateName(self, prefix, name):
        if prefix.StartsWith("^"):
            len = prefix.Length - 1 # ignore ^ character
            if len:
                # "prefix" string is at end
                if self.replacePrefix.IsChecked:                
                    if len < name.Length:
                        name = name.Remove(name.Length - len) # trim off old end   
                name = name + prefix.Substring(1)
        else:
            if self.replacePrefix.IsChecked:
                len = prefix.Length
                if len < name.Length:
                    name = name.Remove(0, len)
            name = prefix + name
        return name.Trim()

    def CancelClicked(self, thisCmd, args):
        thisCmd.CloseUICommand()

    def OkClicked(self, thisCmd, e):
        if self.layerGroupsCtl.SelectedSerial == 0:
            self.layerGroupsCtl.StatusMessage = "Select Layer Group"
            return False

        lgCollection = LayerGroupCollection.GetLayerGroupCollection(self.currentProject, False)
        newLgName = self.newGroupName.Text
        if not newLgName:
            self.layerGroupsCtl.StatusMessage = "New Layer Group Name not defined"
            return False

        if Layer.GetValidAcadLayerName(newLgName) != newLgName:
            self.layerGroupsCtl.StatusMessage = "New Name contains invalid character"
            return False

        if not lgCollection.isUniqueName(newLgName):
            self.layerGroupsCtl.StatusMessage = "New layer group already exists"
            return False
  
        layers = self.currentProject[Project.FixedSerial.LayerContainer]
        layersInGroup = {}
        lGroup = self.layerGroupsCtl.SelectedEntity()
        for layer in layers: 
            # check if layer in selected group
            if layer.LayerGroupSerial != lGroup.SerialNumber:
                continue
            layersInGroup.Add(layer.SerialNumber, 0)

        if not layersInGroup:
            self.layerGroupsCtl.StatusMessage = "No layers found in group"
            return False

        objectsToCopy = []
        wv = self.currentProject[Project.FixedSerial.WorldView]
        self.BuildLayerList(wv, layersInGroup, objectsToCopy)
        if not objectsToCopy:
            self.layerGroupsCtl.StatusMessage = "No objects found"
            return False

        self.layerGroupsCtl.StatusMessage = ""
        prefix = self.groupPrefix.Text

        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(CommandGranularity.Command, self.Caption)
        UIEvents.RaiseBeforeDataProcessing(self, UIEventArgs())
        # pause the graphics so every change doesn't draw
        wv.PauseGraphicsCache(True)
        try:
            self.worldPlotscale = wv.PlotScale
            # the "with" statement will unroll any changes if something go
            # wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                newGroup = lgCollection.Add(clr.GetClrType(LayerGroup))
                newGroup.Name = newLgName
                duplicator = self.currentProject.CreateDuplicator()
                #Copy each entity
                for o in objectsToCopy:
                    finalSite = wv
                    if isinstance(o, IDuplicatableTargetContainer):
                        finalSite = o.CopyTargetContainer
                    else:
                        oldSite = o.GetSite()
                        if isinstance(oldSite, IContainerInternals):
                            finalSite = oldSite

                    #self.currentProject.Duplicate(duplicator, finalSite, o)
                    self.Duplicate(duplicator, finalSite, o)

                IDuplicator.Execute(duplicator)

                # Now update layer
                for o in objectsToCopy:
                    sn = IDuplicatorInternals.GetMappedId(duplicator, o.SerialNumber)
                    if (sn == 0):
                        continue
                    if sn == o.SerialNumber:
                        continue
                    newObj = self.currentProject.Concordance.Lookup(sn)
                    layer = o.Layer
                    l = layersInGroup[layer]
                    if not l:
                        layerObject = layers.Lookup(layer)
                        newLayer = layers.Add(clr.GetClrType(Layer))
                        ln = self.UpdateName(prefix, layerObject.Name)
                        newLayer.Name = layers.GetUniqueLayerName(ln)
                        newLayer.DefaultColor = layerObject.DefaultColor
                        newLayer.LineWeight = layerObject.LineWeight
                        newLayer.LineStyle = layerObject.LineStyle
                        newLayer.LayerGroupSerial = newGroup.SerialNumber
                        l = newLayer.SerialNumber
                        layersInGroup[layer] = l

                    newObj.Layer = l

                # you MUST commit the change to DB or they will all be un-done
                # (inside the "with" scope)
                failGuard.Commit()          

        finally:
            self.currentProject.TransactionManager.AddEndMark(CommandGranularity.Command)
            UIEvents.RaiseAfterDataProcessing(self, UIEventArgs())
            wv.PauseGraphicsCache(False)

        thisCmd.CloseUICommand()

    def Duplicate(self, duplicator, finalSite, o):
        # In general, IronPython works great with .net
        # The TBC api was developed before IronPython existed so there
        # are a few issues.
        # This method works around one of those issues
        types = Array[Type]([clr.GetClrType(IContainerInternals), clr.GetClrType(IDuplicatable)])
        method = clr.GetClrType(IDuplicator).GetMethod("Duplicate", types, None)        
        method.Invoke(duplicator, Array[Object]([finalSite, o]))

    def BuildLayerList(self, container, layersInGroup, objectsToCopy):
            for sn in container:
                # if this is container (but not own coord sys type, like
                # profile)
                if isinstance(sn, clr.GetClrType(IMemberManagement)) and \
                      not isinstance(sn, clr.GetClrType(ISpacialContainer)):
                    self.BuildLayerList(sn, layersInGroup, objectsToCopy)

                if not isinstance(sn, clr.GetClrType(IViewMember)):
                    continue
                if not sn.Layer in layersInGroup:
                    continue
                if not isinstance(sn, clr.GetClrType(IRepresentGraphically)):
                    continue
                if not sn.CanTransform:
                    continue
                if isinstance(sn, clr.GetClrType(ICopy)):
                    if not sn.CanCopy:
                        continue
                objectsToCopy.Add(sn) # this object's layer was in orig layer group
