﻿"""
Macro to allow user to create points by station/offset
"""
# Reference the WPF assemblies
import clr
import InfragisticsDlls
clr.AddReference('IronPython.Wpf')
import wpf
clr.AddReference("PresentationFramework")
clr.AddReference("PresentationCore")
clr.AddReference("windowsbase")
from System import Type, Tuple, Array, Double, String
from System.Windows import  Window, Visibility, MessageBox
from System.Windows.Controls import StackPanel
from System.Windows.Input import Keyboard, AccessKeyManager
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import DataFormats
from System import Environment
from System.IO import StreamReader, Path
from System.Text import StringBuilder
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
from InfragisticsDlls import Workbook, Worksheet, WindowOptions, XamSpreadsheet

from Trimble.Vce.Core.Components import Project
from Trimble.Vce.Core import TransactMethodCall

from Trimble.Vce.Data.RawData import RawDataBlock, RawDataContainer

from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings

from Trimble.Vce.Coordinates import PointCollection, StationOffsetPoint, NEECoordinate, Point

from Trimble.Vce.Interfaces import Client, Point as PointHelper, FeatureCoding
from Trimble.Vce.Interfaces.Units import LinearType, CoordinateOrder, VerticalAngleType

from Trimble.Vce.Alignment import VerticalAlignment, ProfileView, AlignmentGeometryType, SlopeTable
from Trimble.Vce.Alignment.Parameters import *# Station, Elevation, Direction
from Trimble.Vce.Corridor import StationSlopeValueDefinitionTable, StationSlopeValueDefinitionTableEdit, StationSlopeValueDefinitionRow

from Trimble.Vce.Geometry import Point3D, Vector3D, Side
from Trimble.Vce.Geometry.PolySeg.Segment import Segment
clr.AddReference("Trimble.Vce.UI.ConstructionCommands")
from Trimble.Vce.UI.ConstructionCommands import CreateHorizontalAlignmentCmd

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "CreatePointByStationOffset"
    cmdData.CommandName = "CreatePointByStationOffset"
    cmdData.Caption = "_Create Point By Station Offset"
    cmdData.UIForm = "CreatePointByStationOffset"
    try:
        cmdData.Version = 0.92
        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "CAD"
        cmdData.ShortCaption = "Sta/Offset Points"
        cmdData.ToolTipTitle = "Create Point By Station Offset"
        cmdData.ToolTipTextFormatted = "Create points by station & offset"
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2019/12/17/tml-create-point-by-station-offset"
        cmdData.HelpFile = cmdData.MacroInfo
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass


class CreatePointByStationOffset(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\CreatePointByStationOffset.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject
        self.macroFileFolder = macroFileFolder
        self.pointCollection = PointCollection.ProvidePointCollection(currentProject)
        self.macroName = "Create Point By Station Offset"
        self.tmlFileFolder = macroFileFolder
        
    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Apply"
        buttons[1].Content = "Close"
        self.okBtn = buttons[0]
        self.Caption = cmd.Command.Caption
        self.alignmentCtl.FilterByEntityTypes = Array[Type]([clr.GetClrType(ProfileView)])
        self.alignmentCtl.NewCmdCallback = CreateHorizontalAlignmentCmd.RunThisCmd
        self.alignmentCtl.AllowNone = False
        self.alignmentCtl.ValueChanged += self.AlignmentChanged
        self.AlignmentChanged(None, None) # this will cause the station controls to get ref hal
        settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
        l = settings.GetUInt32("CreatePointByStationOffset.Layer", 0)
        if self.currentProject.Concordance.Lookup(l):
            self.layerCtl.SelectedSerialNumber = l
        self.nameCtl.SelectedName = settings.GetString("CreatePointByStationOffset.Name",
                     PointHelper.Helpers.Increment("0", self.pointCollection))
        self.createDependentCtl.IsChecked = settings.GetBoolean("CreatePointByStationOffset.CreateDependent", False)

    def AlignmentChanged(self, sender, e):
        hal = self.alignmentCtl.SelectedEntity()
        self.staCtl.StationProvider = hal
        self.offsetCtl.StationProvider = hal

    def CancelClicked(self, sender, args):
        settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
        settings.SetUInt32("CreatePointByStationOffset.Layer", self.layerCtl.SelectedSerialNumber)
        settings.SetString("CreatePointByStationOffset.Name", self.nameCtl.SelectedName)
        settings.SetBoolean("CreatePointByStationOffset.CreateDependent", self.createDependentCtl.IsChecked)
        sender.CloseUICommand()

    def OkClicked(self, sender, e):
        hal = self.alignmentCtl.SelectedSerial 
        if hal== 0:
            return False
        sta = self.staCtl.Distance
        if Double.IsNaN(sta):
            self.staCtl.StatusMessage = "Enter valid station"
            return
        offset = self.offsetCtl.Offset
        if Double.IsNaN(offset):
            self.offsetCtl.StatusMessage = "Enter valid offset"
            return
        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)
        try:
            # unroll any changes if something goes wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
  
                self.CreatePoint(self.nameCtl.SelectedName, self.layerCtl.SelectedSerialNumber, sta, 
                         offset, self.elevCtl.Elevation, self.codeCtl.Text, hal)
                self.nameCtl.SelectedName = PointHelper.Helpers.Increment(self.nameCtl.SelectedName, self.pointCollection)
                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

        Keyboard.Focus(self.okBtn)
        if self.staCtl.AutoTabExtSkip:
            Keyboard.Focus(self.offsetCtl)
        else:
            Keyboard.Focus(self.staCtl)

    def CreatePoint(self, id, layer, station, offset, elev, fCode, hal):    
        if self.createDependentCtl.IsChecked:
            p = self.pointCollection.Add(clr.GetClrType(StationOffsetPoint))
            p.AddCoordinate( NEECoordinate(0, 0, elev)) # only need Z as X,Y are calculated        
            p.Station = station
            p.Offset = offset
            p.ReferenceGeometrySerial = hal
        else:
            p = self.pointCollection.Add(clr.GetClrType(Point))
            coord = self.ComputeCoordinate()
            coord.Z = elev
            p.AddCoordinate(NEECoordinate(coord))

        p.PointID = id
        p.Layer = layer
        if fCode:
            f = FeatureCoding.IFeatureCodeHost.CreateAndHost(p)
            f.Code = fCode

    def ComputeCoordinate(self):
        p = Point3D(0,0)
        line = self.alignmentCtl.SelectedEntity()
        sta = self.staCtl.Distance
        offset = self.offsetCtl.Offset
        hPoly=line.ComputePolySeg()
        if not hPoly or hPoly.NumberOfNodes < 2:
            return p
        seg = clr.Reference[Segment]()
        t = clr.Reference[float]()
        pcl = clr.Reference[Point3D]()
        v = clr.Reference[Vector3D]()
        if not hPoly.FindPointFromStation(sta, seg, t, pcl, v):
            return p
        v = v.Value
        v.To2D()
        v.Normalize()
        p = pcl.Value + offset * v;
        return p