﻿"""
Macro to allow user to add extra stations to alignment
"""
# Reference the WPF assemblies
import clr
import InfragisticsDlls
clr.AddReference('IronPython.Wpf')
import wpf
clr.AddReference("PresentationFramework")
clr.AddReference("PresentationCore")
clr.AddReference("windowsbase")
from System import Type, Tuple, Array, Double, String
from System.Collections.Generic import List
from System.Windows import  Window, Visibility, MessageBox
from System.Windows.Controls import StackPanel, CheckBox
from System.Windows.Input import Keyboard, AccessKeyManager
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import DataFormats
from System import Environment
from System.IO import StreamReader, Path
from System.Text import StringBuilder
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
from InfragisticsDlls import Workbook, Worksheet, WindowOptions, XamSpreadsheet

from Trimble.Vce.Core.Components import Project
from Trimble.Vce.Core import TransactMethodCall, ModelEvents

from Trimble.Vce.Data.RawData import RawDataBlock, RawDataContainer

from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings

from Trimble.Vce.Coordinates import PointCollection, StationOffsetPoint, NEECoordinate, Point

from Trimble.Vce.Interfaces import Client, Point as PointHelper, FeatureCoding
from Trimble.Vce.Interfaces.Units import LinearType, CoordinateOrder, VerticalAngleType
try:
    from Trimble.Sdk.UI import InputMethod
except:
    from Trimble.Sdk.Interfaces.UI import InputMethod

from Trimble.Vce.Alignment import VerticalAlignment, ProfileView, AlignmentGeometryType, SlopeTable
from Trimble.Vce.Alignment.Parameters import *# Station, Elevation, Direction
from Trimble.Vce.Corridor import StationSlopeValueDefinitionTable, StationSlopeValueDefinitionTableEdit, StationSlopeValueDefinitionRow

from Trimble.Vce.Geometry import Point3D, Vector3D, Side
from Trimble.Vce.Geometry.PolySeg.Segment import Segment
clr.AddReference("Trimble.Vce.UI.ConstructionCommands")
from Trimble.Vce.UI.ConstructionCommands import CreateHorizontalAlignmentCmd

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "DefineExtraStations"
    cmdData.CommandName = "DefineExtraStations"
    cmdData.Caption = "_Define Extra Stations"
    cmdData.UIForm = "DefineExtraStations"
    try:
        cmdData.Version = 0.92
        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "CAD"
        cmdData.ShortCaption = "Sta/Offset Points"
        cmdData.ToolTipTitle = "Create Point By Station Offset"
        cmdData.ToolTipTextFormatted = "Create points by station & offset"
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2019/12/17/tml-create-point-by-station-offset"
        cmdData.HelpFile = cmdData.MacroInfo
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass


class DefineExtraStations(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\DefineExtraStations.xaml") as s:
            wpf.LoadComponent(self, s)

        self.currentProject = currentProject
        self.macroFileFolder = macroFileFolder
        self.pointCollection = PointCollection.ProvidePointCollection(currentProject)
        self.macroName = "xxxx"
        self.tmlFileFolder = macroFileFolder
        
    def OnLoad(self, cmd, buttons, event):
        ModelEvents.EntityEdited += self.ModelEvents_EntityEdited
        buttons[0].Content = "Apply"
        buttons[1].Content = "Close"
        self.okBtn = buttons[0]
        self.Caption = cmd.Command.Caption
        self.staUnits = self.currentProject.Units.Station
        self.staProperties = self.staUnits.Properties
        self.alignmentCtl.FilterByEntityTypes = Array[Type]([clr.GetClrType(ProfileView)])
        self.alignmentCtl.NewCmdCallback = CreateHorizontalAlignmentCmd.RunThisCmd
        self.alignmentCtl.AllowNone = False
        self.alignmentCtl.ValueChanged += self.AlignmentChanged
        self.AlignmentChanged(None, None) # this will cause the station controls to get ref hal

    def Dispose(self, thisCmd, disposing):
        ModelEvents.EntityEdited -= self.ModelEvents_EntityEdited

    def ModelEvents_EntityEdited(self, obj, eArgs):
        serial = eArgs.EntitySerialNumber
        if serial == self.alignmentCtl.SelectedSerial:
            # rebuild station list
            self.LoadStationList()

    def LoadStationList(self):
        self.ctrlStationList.Items.Clear()
        if not self.alignmentCtl.SelectedSerial:
            return
        hal = self.alignmentCtl.SelectedEntity()
        extra = hal.ExtraStations
        if not extra:
            return
        extra.Sort()
        for station in extra:
            station = self.staUnits.Format(station, self.staProperties, hal)
            c = CheckBox()
            c.Content=station
            self.ctrlStationList.Items.Add(c)

    def delSelected_Click(self, sender, e):
        if not self.alignmentCtl.SelectedSerial:
            return
        hal = self.alignmentCtl.SelectedEntity()
        extra = hal.ExtraStations
        if not extra:
            return
        self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)
        try:
            # unroll any changes if something goes wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                for i in range(self.ctrlStationList.Items.Count):
                    n = self.ctrlStationList.Items.Count - i -1
                    if self.ctrlStationList.Items[n].IsChecked:
                        extra.RemoveAt(n)

                hal.ExtraStations = extra
  
                failGuard.Commit()
        except:
            pass
        finally:
            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

    def AlignmentChanged(self, sender, e):
        hal = self.alignmentCtl.SelectedEntity()
        self.staCtl.StationProvider = hal
        self.LoadStationList()

    def staCtl_ValueChanged(self, sender, e):
        if e.Cause == InputMethod.Mouse:     
            self.OkClicked(None, None)

    def CancelClicked(self, sender, args):
        sender.CloseUICommand()

    def OkClicked(self, sender, e):
        hal = self.alignmentCtl.SelectedSerial 
        if hal== 0:
            return False
        sta = self.staCtl.Distance
        if Double.IsNaN(sta):
            self.staCtl.StatusMessage = "Enter valid station"
            return

        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)
        try:
            # unroll any changes if something goes wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                hal = self.alignmentCtl.SelectedEntity()
                extra = hal.ExtraStations
                if not extra:
                    extra = List[Double]()
                extra.Add(sta)
                hal.ExtraStations = extra
  
                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

        Keyboard.Focus(self.okBtn)
        Keyboard.Focus(self.staCtl)