﻿"""
Macro to allow user to edit slope distances in a spreadsheet
"""
# Reference the WPF assemblies
import clr
import InfragisticsDlls
clr.AddReference('IronPython.Wpf')
import wpf
clr.AddReference("PresentationFramework")
clr.AddReference("PresentationCore")
clr.AddReference("windowsbase")
from System import Type, Tuple, Array, Double, String
from System.Windows import  Window, Visibility, MessageBox
from System.Windows.Controls import StackPanel
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import DataFormats
from System import Environment
from System.Windows.Input import Keyboard
from System.IO import StreamReader, Path
from System.Text import StringBuilder
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
from InfragisticsDlls import Workbook, Worksheet, WindowOptions, XamSpreadsheet

from Trimble.Vce.Core.Components import Project
from Trimble.Vce.Core import TransactMethodCall

from Trimble.Vce.Coordinates import PointCollection, NEECoordinate, Point

from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings

from Trimble.Vce.Interfaces import ProgressBar, Point as PointHelper
from Trimble.Vce.Interfaces import Client
from Trimble.Vce.Interfaces.Units import LinearType, CoordinateOrder, VerticalAngleType, AzimuthType, AngularType
try:
    from Trimble.Sdk.UI import InputMethod
except:
    from Trimble.Sdk.Interfaces.UI import InputMethod

from Trimble.Vce.Alignment import VerticalAlignment, ProfileView, AlignmentGeometryType, SlopeTable
from Trimble.Vce.Alignment.Parameters import *# Station, Elevation, Direction
from Trimble.Vce.Alignment.Linestring import Linestring,VerticalElementFactory, IVerticalNoCurve, IStationVerticalLocation
from Trimble.Vce.Corridor import StationSlopeValueDefinitionTable, StationSlopeValueDefinitionTableEdit, StationSlopeValueDefinitionRow

from Trimble.Vce.Geometry import Point3D, Intersections, Vector3D, Side
from Trimble.Vce.Geometry.PolySeg import PolySeg
from Trimble.Vce.Geometry.PolySeg.Segment import Segment
clr.AddReference("Trimble.Vce.UI.ConstructionCommands")
from Trimble.Vce.UI.ConstructionCommands import CreateHorizontalAlignmentCmd

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "DivideLineBySlopeDistances"
    cmdData.CommandName = "DivideLineBySlopeDistances"
    cmdData.Caption = "_Divide Line By Slope Distances"
    cmdData.UIForm = "DivideLineBySlopeDistances"

    try:
        cmdData.Version = 1.02
        cmdData.MacroAuthor = "Trimble"
    except:
        pass
    try:
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

class DivideLineBySlopeDistances(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\DivideLineBySlopeDistances.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject
        self.macroFileFolder = macroFileFolder
        self.macroName = "Divide Line By Slope Distances"

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Apply"
        buttons[1].Content = "Close"
        self.okBtn = buttons[0]
        self.Caption = cmd.Command.Caption
        self.refLine.AddTypeFilter(clr.GetClrType(Linestring))
        self.pointCollection = PointCollection.ProvidePointCollection(self.currentProject)

        settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
        l = settings.GetUInt32("DivideLineBySlopeDistances.Layer", 0)
        if self.currentProject.Concordance.Lookup(l):
            self.layerCtl.SelectedSerialNumber = l
        self.nameCtl.SelectedName = settings.GetString("DivideLineBySlopeDistances.Name", "1")

    def CancelClicked(self, sender, args):
        sender.CloseUICommand()

    def refLine_ValueChanged(self, sender, e):
        l = self.refLine.Entity
        if l and  e.Cause == InputMethod.Mouse:     
            self.OkClicked(None, None)

    def OkClicked(self, sender, e):
        line = self.refLine.Entity
        if not line:
            self.refLine.StatusMessage= "No line selected"
            return

        distances = getSavedDistances(self.currentProject)
        if not len(distances):
            self.refLine.StatusMessage= "No distances defined"
            return
        Keyboard.Focus(self.okBtn)
        self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)
        try:
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                hPoly=line.ComputePolySeg()
                if not hPoly or hPoly.NumberOfNodes < 2:
                    self.refLine.StatusMessage= "Line not defined"
                    return
                vPoly=line.ComputeVerticalPolySeg()
                if not vPoly or vPoly.NumberOfNodes < 2:
                    self.refLine.StatusMessage= "No vertical defined"
                    return
                vPoly.ComputeStationing()
                currentDist = 0.0
                self.CreatePointAtVerticalStation(currentDist, hPoly, vPoly)
                for d in distances:
                    currentDist += d
                    self.CreatePointAtVerticalStation(currentDist, hPoly, vPoly)

                settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
                settings.SetUInt32("DivideLineBySlopeDistances.Layer", self.layerCtl.SelectedSerialNumber)
                settings.SetString("DivideLineBySlopeDistances.Name", self.nameCtl.SelectedName)
                # you MUST commit the change to DB or they will all be un-done (roll-back)
                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

        Keyboard.Focus(self.refLine)
        return

    def CreatePointAtVerticalStation(self, staOnVal, hPoly, vPoly):
        seg = clr.Reference[Segment]()
        t = clr.Reference[float]()
        pcl = clr.Reference[Point3D]()
        v = clr.Reference[Vector3D]()
        if not vPoly.FindPointFromStation(staOnVal, seg, t, pcl, v):
            return False
        sta = pcl.Value.X
        elev = pcl.Value.Y
        if not hPoly.FindPointFromStation(sta, seg, t, pcl, v):
            return False
        coord = pcl.Value
        coord.Z = elev
        self.CreatePoint(coord)
        return True

    def CreatePoint(self, coord):
        p = self.pointCollection.Add(clr.GetClrType(Point))
        p.PointID = self.nameCtl.SelectedName
        self.nameCtl.SelectedName = PointHelper.Helpers.Increment(self.nameCtl.SelectedName, None)
        p.AddCoordinate( NEECoordinate(coord))
        p.Layer = self.layerCtl.SelectedSerialNumber
    
    def configDistances_Click(self, sender, e):
        try:
            rtn = DivideLineBySlopeDistancesDialog(self.currentProject, self.Caption, self.macroFileFolder).ShowDialog()
        except Exception as e:
            MessageBox.Show(str(e), "Macro threw exception")

def getSavedDistances(project):
        settings = ConstructionCommandsSettings.ProvideObject(project)
        count = settings.GetInt32("DivideLineBySlopeDistances.Count", 0)
        distances = []
        for i in range(count):
            d = settings.GetDouble("DivideLineBySlopeDistances.Distance" + str(i), Double.NaN)
            if not Double.IsNaN(d):
                distances.append(d)
        return distances

def setSavedDistances(project, distances):
        settings = ConstructionCommandsSettings.ProvideObject(project)
        count = 0        
        for i in range(len(distances)):
            d = distances[i]
            if not Double.IsNaN(d):
                settings.SetDouble("DivideLineBySlopeDistances.Distance"+ str(i), d)
                count += 1
        settings.SetInt32("DivideLineBySlopeDistances.Count", count)

class DivideLineBySlopeDistancesDialog(Window): # this inherits from the WPF window control
    def __init__(self, currentProject, Caption, macroFileFolder):
        with StreamReader(macroFileFolder + r"\DivideLineBySlopeDistancesDialog.xaml") as s:
            wpf.LoadComponent(self, s) 

        self.Caption = Caption
        self.currentProject = currentProject
        self.macroFileFolder = macroFileFolder
        self.lunits = currentProject.Units.Linear
        self.wb = self.Spreadsheet.Workbook
        sheet = self.wb.WindowOptions.SelectedWorksheet
        self.LoadSegments(sheet)
        self.SaveBtn.Click+=self.Button_Save_Click
        self.CloseBtn.Click += self.Button_Click_Cancel

    def ConvertToDisplay(self, value):
        return self.lunits.Convert(value, LinearType.Display)

    def ConvertToMeters(self, sheet, row, value):
        try:
            if isinstance(value, Double):
                return self.lunits.Convert(LinearType.Display, value, LinearType.Meter)
            else:
                if value == None:
                    return Double.NaN
                # string
                s = str(value).strip()
                if value == self.currentProject.Units.UndefinedValue:
                    return Double.NaN
                # convert to string so we can handle both strings and double
                return self.lunits.Parse(s)
        except:
            pass
        raise Exception('Parse error Sheet:' + sheet.Name + ' Row:' + str(row+1))

    def LoadSegments(self, sheet):  
        # set number of decimals for first column
        show3 = String.Format("{0:0.000}", 0.0)
        distances = getSavedDistances(self.currentProject)
        row = 0
        for i in range(len(distances)):
            cells = sheet.Rows[row].Cells
            cells[0].Value = self.ConvertToDisplay(distances[i])
            row = row + 1

    def SaveSegments(self, sheet):
        i = 0
        distances = []
        for row in sheet.Rows:
            cells = row.Cells
            if (not cells[0].GetText().Length > 0):
                    continue
            d = self.ConvertToMeters(sheet, row.Index, cells[0].Value)
            if not Double.IsInfinity(d):
                i += 1
                distances.append(d)
        setSavedDistances(self.currentProject, distances)

    def Button_Click_Cancel(self, sender, e):
        self.Close()

    def Button_Save_Click(self, sender, e):
        try:
            self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)
       
            # the "with" statement will unroll any changes if something go
            # wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                for i in range(1):
                    sheet = self.Spreadsheet.Workbook.Worksheets[i]
                    self.SaveSegments(sheet)
                failGuard.Commit()

            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)
            self.Close()

        except Exception as e:
            MessageBox.Show(str(e), "Macro threw exception")
