﻿"""
Sample TBC macro to explode line segments into seperate lines
"""
import clr
# Reference the WPF assemblies
clr.AddReference('IronPython.Wpf')
import wpf
from System import Math, Array, Type
from System.IO import StreamReader
from System.Windows.Controls import StackPanel
from System.Windows.Input import Keyboard
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap

from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle, BlockCollection, BlockView, \
        LayerGroupCollection, LayerGroup
from Trimble.Vce.Core import TransactMethodCall
from Trimble.DiskIO import OptionsManager

from Trimble.Vce.Geometry import Point3D, Vector3D, Side
from Trimble.Vce.Geometry.PolySeg import PolySeg
from Trimble.Vce.Geometry.PolySeg.Node import Spiral

from Trimble.Vce.Interfaces.SnapIn import IPolyseg, IHaveUcs, ISnapIn, ILinestringModifier, IName
from Trimble.Vce.Interfaces.Client import CommandGranularity
clr.AddReference("Trimble.Vce.UI.Controls")
from Trimble.Vce.Core import GlobalSelection

from Trimble.Vce.Alignment import ProfileView
from Trimble.Vce.Alignment.Linestring import Linestring, Polygon
clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import SelectionContextMenuHandler
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents
try:
    from Trimble.Sdk.UI import UIEventArgs
except:
    from Trimble.Sdk.Interfaces.UI import UIEventArgs   


# define the command key and name
def Setup(cmdData, macroFileFolder):
    cmdData.Key = "ExplodeLines"
    cmdData.CommandName = "ExplodeLines"
    cmdData.Caption = "_Explode Lines"
    cmdData.UIForm = "ExplodeLines"

    try:
        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "CAD"
        cmdData.ShortCaption = "Explode lines"
        cmdData.ToolTipTitle = "Explode Lines"
        cmdData.ToolTipTextFormatted = "Explode selected line geometry into Linestring objects."

        cmdData.Version = 1.09
        cmdData.MacroAuthor = "Trimble"

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass 

class ExplodeLines(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\ExplodeLines.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject
        self.macroName = "Explode Lines"
        self.tmlFileFolder = macroFileFolder   

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Apply"
        buttons[1].Content = "Close"
        self.Caption = cmd.Command.Caption
        self.objs.IsEntityValidCallback = self.IsValid
        optionMenu = SelectionContextMenuHandler()
        # remove options that don't apply here
        optionMenu.ExcludedCommands = "SelectObservations | SelectPoints | SelectDuplicatePoints"
        self.objs.ButtonContextMenu = optionMenu
        self.polyType = clr.GetClrType(IPolyseg)
        self.lType = clr.GetClrType(Linestring)
        self.lMod = clr.GetClrType(ILinestringModifier)
        self.Profile = clr.GetClrType(ProfileView)
        self.ucs = clr.GetClrType(IHaveUcs)

        self.explodeDeflection.IsChecked = OptionsManager.GetBool("ExplodeLines:explodeOption", False)
        self.explodeAll.IsChecked = not self.explodeDeflection.IsChecked
        self.maxDeflectionAngle.Angle = OptionsManager.GetDouble("ExplodeLines:maxDeflectionAngle", Math.PI / 4)
        
        self.relayerCB.IsChecked = OptionsManager.GetBool("ExplodeLines:relayerCB", False)
        self.topLayer.Text = OptionsManager.GetString("ExplodeLines:topLayer", "{L}-Top")
        self.bottomLayer.Text = OptionsManager.GetString("ExplodeLines:bottomLayer", "{L}-Bottom")
        self.verticalLayer.Text = OptionsManager.GetString("ExplodeLines:verticalLayer", "{L}-Vert")

    def CancelClicked(self, cmd, args):
        cmd.CloseUICommand()

    def IsValid(self, serial):
        o = self.currentProject.Concordance.Lookup(serial)
        return self.IsValidObj(o)

    def IsValidObj(self, obj):
        if isinstance(obj, self.Profile):
            return False
        if isinstance(obj, self.polyType):
            return True
        return False

    def UpdateLayer(self, ls, layers, layerString):
        layer = layers.Lookup(ls.Layer)
        if not layer:
            return
        newLayerName = layerString.Replace("{L}", layer.Name)
        layerNew = layers.GetLayer(newLayerName)
        if layerNew:
            ls.Layer = layerNew.SerialNumber
            return
        # layer doesn't exist
        newLayer = layers.Add(clr.GetClrType(Layer))
        newLayer.Name = newLayerName
        newLayer.DefaultColor = layer.DefaultColor
        newLayer.LineWeight = layer.LineWeight
        newLayer.LineStyle = layer.LineStyle
        newLayer.LayerGroupSerial = layer.LayerGroupSerial
        ls.Layer = newLayer.SerialNumber

    def CreateLinestring(self, container, oldLine, polyseg, vPoly):
        ls = container.Add(self.lType)
        if isinstance(oldLine, self.ucs):
            ls.Normal = oldLine.Normal
        ls.SetName = IName.Name.__get__(oldLine)
        ls.Layer = oldLine.Layer
        if self.relayerCB.IsChecked and self.polysegClosed:
            x1 = polyseg.FirstNode.Point
            x2 = polyseg.LastNode.Point
            # if dy is 10X dx, treat as vertical
            if Math.Abs(x2.X - x1.X) < Math.Abs(x2.Y - x1.Y) / 10:
                layerString = self.verticalLayer.Text
            elif self.polyClockwise:
                if x2.X > x1.X:
                    #top
                    layerString = self.topLayer.Text
                else:
                    layerString = self.bottomLayer.Text
            else:
                if x2.X < x1.X:
                    #top
                    layerString = self.topLayer.Text
                else:
                    layerString = self.bottomLayer.Text
            self.UpdateLayer(ls, self.layerCollection, layerString)
                    
        ls.Color = oldLine.Color
        ls.Weight = oldLine.Weight
        try:
            # version 5.10 doesn't handle polysegs where first sta is not 0. Shift vertical to fix
            if vPoly and polyseg.FirstNode.Station != 0:
                sta = polyseg.FirstNode.Station
                polyseg.ComputeStationing()
                vPoly = vPoly.Clone()
                vPoly.Translate(None, None, Vector3D(-sta, 0))
            ls.Append(polyseg, vPoly, False, False)
        except:
            # some objects with funny UCS throw when trying to append to
            # linestring
            container.Remove(ls.SerialNumber)
            return None
        try:
            # version 5.2 can trim extra vertical elements
            ls.TrimExtraVerticalElements()
        except:
            pass
        return ls

    def CheckDeflection(self, v1,v2):
        if v1.Is2D:
            v1.Z = 0
        if v2.Is2D:
            v2.Z = 0
        v1.Normalize()
        v2.Normalize()
        (dot, v1, v2) = Vector3D.DotProduct(v1, v2)
        # check for round-off
        if dot > 1.0:
            dot = 1.0
        if dot < -1.0:
            dot = -1.0
        angle = Math.Acos(dot)
        return angle <= self.maxDeflectionAngle.Angle

    def OkClicked(self, cmd, e):
        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(CommandGranularity.Command, self.Caption)
        wv = self.currentProject[Project.FixedSerial.WorldView]
        self.layerCollection = self.currentProject[Project.FixedSerial.LayerContainer]
        UIEvents.RaiseBeforeDataProcessing(self, UIEventArgs())
        # pause the graphics so every change doesn't draw
        wv.PauseGraphicsCache(True)
        try:            
            # the "with" statement will unroll any changes if something go
            # wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                # gete the current list so we can clear global selection
                entities = Array[ISnapIn](self.objs.SelectedMembers(self.currentProject))
                GlobalSelection.Clear()
                for o in entities:
                    if not self.IsValidObj(o):
                        continue
                    polySeg = o.ComputePolySeg()
                    if not polySeg:
                        continue
                    vPoly = o.ComputeVerticalPolySeg()
                    polySeg.DeleteZeroLengthSegments()
                    if polySeg.NumberOfNodes < 3:
                        continue          
                    container = o.GetSite()
                    self.polysegClosed = polySeg.IsClosed
                    if self.polysegClosed :
                        self.polyClockwise = polySeg.IsClockWise()
                    firstLine = None
                    lastSeg = None
                    newPoly = None
                    for node in polySeg.Nodes():
                        seg = polySeg.CreateSegment(node)
                        if seg:
                            if self.explodeDeflection.IsChecked:
                                if lastSeg:
                                    (b,vLast) = lastSeg.Slope(1.0) # get direction at end of last segment
                                    (b,v) = seg.Slope(0.0)
                                    if self.CheckDeflection(vLast, v):
                                        newPoly.Add(seg)
                                        lastSeg = seg
                                        continue
                            if newPoly:
                                newPoly.ComputeStationing()
                                l = self.CreateLinestring(container, o, newPoly, vPoly)
                                if not firstLine:
                                    firstLine = l
                            newPoly = PolySeg()
                            newPoly.OCSToWorld = polySeg.OCSToWorld
                            newPoly.BeginStation = node.Station
                            newPoly.Add(seg)      
                            lastSeg = seg
                    if newPoly:
                        newPoly.ComputeStationing()
                        lastLine = self.CreateLinestring(container, o, newPoly, vPoly)
                        if self.explodeDeflection.IsChecked and firstLine:
                            if polySeg.IsClosed:
                                # check if we need to join first/last lines
                                polySeg = firstLine.ComputePolySeg()
                                (b,vLast) = polySeg.FirstSegment.Slope(0.0)
                                (b,v) = newPoly.LastSegment.Slope(1.0)
                                if self.CheckDeflection(v, vLast):
                                    Linestring.Join(lastLine, firstLine)
                    if self.checkBox.IsChecked:
                        container.Remove(o.SerialNumber)

                # you MUST commit the change to DB or they will all be un-done
                # (inside the "with" scope)
                failGuard.Commit()

        finally:
            UIEvents.RaiseAfterDataProcessing(self, UIEventArgs())
            wv.PauseGraphicsCache(False)
            self.currentProject.TransactionManager.AddEndMark(CommandGranularity.Command)

        OptionsManager.SetValue("ExplodeLines:explodeOption", self.explodeDeflection.IsChecked)
        OptionsManager.SetValue("ExplodeLines:maxDeflectionAngle", self.maxDeflectionAngle.Angle)

        OptionsManager.SetValue("ExplodeLines:relayerCB", self.relayerCB.IsChecked)
        OptionsManager.SetValue("ExplodeLines:topLayer", self.topLayer.Text)
        OptionsManager.SetValue("ExplodeLines:bottomLayer", self.bottomLayer.Text)
        OptionsManager.SetValue("ExplodeLines:verticalLayer", self.verticalLayer.Text)

        Keyboard.Focus(self.objs)

