import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System import Math, Action, Func
from System.Text import Encoding
from System.Collections.Generic import List
from System.Windows import Visibility
from System.Windows.Controls import StackPanel
from System.Windows.Input import Keyboard, AccessKeyManager
from System import Type, Tuple, Array, Double
from System.IO import StreamReader, MemoryStream
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components.TextStyle import TextHeightMode, TextJustification
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle
from Trimble.Vce.SurveyCAD import ProjectLineStyle

from Trimble.DiskIO import SearchPath, OptionsManager
from Trimble.Vce.Alignment.Linestring import Linestring,ElementFactory, IStraightSegment, IXYZLocation,ITangentArcSegment, ArcTangentType
from Trimble.Vce.Alignment.Parameters import Bearing, Elevation

from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings

from Trimble.Vce.ForeignCad import TextEntity, Leader, MText
from Trimble.Vce.Geometry import Point3D

from Trimble.Vce.Interfaces.Units import LinearType
from Trimble.Vce.Interfaces import Client
from Trimble.Vce.Interfaces.SnapIn import IPolyseg, IViewMember, ISettableName, IName, IViewSite, ISnapInHelpers
clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import ViewHelper
clr.AddReference("Trimble.Vce.UI.Wrapper")
from Trimble.Vce.UI.Wrapper import CADWrapperHelper
try:
    from Trimble.Sdk.UI import InputMethod
except:
    from Trimble.Sdk.Interfaces.UI import InputMethod

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "IncrementText"
    cmdData.CommandName = "IncrementText"
    cmdData.Caption = "_Increment Text"
    cmdData.UIForm = "IncrementText"
    try:
        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "CAD"
        cmdData.ShortCaption = "Increment Text"

        cmdData.Version = 1.08
        cmdData.MacroAuthor = "Trimble"
        cmdData.ToolTipTitle = "Increment Text"
        cmdData.ToolTipTextFormatted = "A simplified version of the Create Linestring command."
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2020/01/14/tml-increment-text"
        cmdData.HelpFile = cmdData.MacroInfo
    except:
        pass
    try:
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

class IncrementText(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\IncrementText.xaml") as s:
            wpf.LoadComponent(self, s)

        self.currentProject = currentProject
        self.macroName = "Increment Text"
        self.tmlFileFolder = macroFileFolder

        self.lunits = self.currentProject.Units.Linear
        self.staUnits = self.currentProject.Units.Station
        self.staProperties = self.staUnits.Properties

        settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
        l = settings.GetUInt32("IncrementText.Layer", 0)
        if self.currentProject.Concordance.Lookup(l):
            self.layerCtl.SelectedSerialNumber = l
            
        ts = settings.GetUInt32("IncrementText.TextStyle", 0)
        if self.currentProject.Concordance.Lookup(ts):
            self.labelStyleCtl.SelectedSerialNumber = ts
        self.currentCtl.Value = settings.GetDouble("IncrementText.CurrentValue", Double.NaN)
        self.incrementCtl.Value = settings.GetDouble("IncrementText.IncrementValue", 10.0)
        self.templateTextCtl.Text = settings.GetString("IncrementText.Template", "{}")
        self.lastPoint = Point3D.Undefined
        self.valueChangedByUser = False

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Apply"
        buttons[1].Content = "Close"
        self.Caption = cmd.Command.Caption

    def coordChanged(self, ctrl, e):
        # set keyboard focus if change was due to mouse pick
        if e.Cause == InputMethod.Mouse:   
            self.OkClicked(None, None)

    def currentCtl_ValueChanged(self, ctrl, e):
        self.updateResult()        

    def resultTextCtl_TextChanged(self, ctrl, e):
        self.valueChangedByUser = True

    def resultTextCtl_GotFocus(self, ctrl, e):
        self.resultTextCtl.SelectAll()

    def textChanged(self, ctrl, e):
        self.updateResult()

    def updateResult(self):
        c = self.currentCtl.Value
        if Double.IsNaN(c) or Math.Abs(c) > 100000000.0:
            return
        txt = self.templateTextCtl.Text
        if txt.Contains("{}"):        
            self.resultTextCtl.Text = txt.Replace("{}", str(int(c)))
        elif txt.Contains("{s}"):
            station = self.lunits.Convert(LinearType.Display, c, LinearType.Meter)
            station = self.staUnits.Format(station, self.staProperties, None)
            self.resultTextCtl.Text = txt.Replace("{s}", station)            
        elif txt.Contains("{0}"):        
            self.resultTextCtl.Text = txt.Replace("{0}", str(int(c)))
        else:
            for n in range(9):
                sn = str(n)
                s = "{" + sn + "}"
                if txt.Contains(s):
                    sn = "{0:." + sn + "f}"
                    self.resultTextCtl.Text = txt.Replace(s, sn.format(c))
                    self.valueChangedByUser = False
                    return
            self.resultTextCtl.Text = txt
        self.valueChangedByUser = False

    def ComputeTextHeight(self, textStyle, viewContainer, worldPlotScale):
        height = textStyle.Height
        if isinstance(viewContainer, clr.GetClrType(IViewSite)):
            plotScale = viewContainer.PlotScale
        else:
            csd,view = ISnapInHelpers.GetViewSite(viewContainer)
            plotScale = view.PlotScale
        
        if plotScale < 1.1:
            # sheet view
            if textStyle.HeightMode == TextHeightMode.Ground:
                height = height / worldPlotScale
            else:
                height /= 1000.0
        else:
            if textStyle.HeightMode == TextHeightMode.Paper:
                height = height * plotScale / 1000
        return height

    def CancelClicked(self, thisCmd, args):
        thisCmd.CloseUICommand()

    def OkClicked(self, thisCmd, e):
        currentValue = self.currentCtl.Value
        if Double.IsNaN(currentValue):
            self.currentCtl.StatusMessage = "Enter valid start value"
            Keyboard.Focus(self.currentCtl)
            return False
        self.currentCtl.StatusMessage = ""

        incValue = self.incrementCtl.Value
        if Double.IsNaN(incValue):
            self.incrementCtl.StatusMessage = "Enter valid increment value"
            Keyboard.Focus(self.incrementCtl)
            return False
        self.incrementCtl.StatusMessage = ""

        ctrl = self.coordCtl
        coord = ctrl.Coordinate
        if coord.IsUndefined or not ctrl.ResultCoordinateSystem:
            self.coordCtl.StatusMessage = "No coordinate defined"
            Keyboard.Focus(ctrl)
            return False
        self.coordCtl.StatusMessage = ""
        if (self.lastPoint.IsDuplicate2D(coord)):
            ctrl.Clear()
            Keyboard.Focus(self.resultTextCtl)
            return False
        self.lastPoint = coord

        text = self.resultTextCtl.Text
        if not text:
            self.incrementCtl.StatusMessage = "Enter valid increment value"
            return False

        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)
        worldPlotScale = self.currentProject[Project.FixedSerial.WorldView].PlotScale
        try:
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                crd = ctrl.Coordinate
                container = ViewHelper.GetContainerForView(ctrl.ClickWindow) # usually WorldView

                textObj = container.Add(clr.GetClrType(MText))
                textObj.Point0 = crd
                textStyle = self.labelStyleCtl.SelectedSnapIn
                textObj.TextString = text
                textObj.Layer = self.layerCtl.SelectedSerialNumber
                textObj.TextStyleSerial = textStyle.SerialNumber
                textObj.WidthFactor = textStyle.WidthFactor
                textObj.AddWhiteOut = textStyle.WhiteOut
                textObj.AutoFlip = textStyle.AutoFlip
                textObj.ObliqueAngle = textStyle.ObliqueAngle
                textObj.Height = self.ComputeTextHeight(textStyle, container, worldPlotScale)
                textObj.AttachPoint = CADWrapperHelper.ConvertTextJustificationToAttachmentPoint(textStyle.Justification)

                settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
                settings.SetString("IncrementText.Template", self.templateTextCtl.Text)
                settings.SetUInt32("IncrementText.Layer", self.layerCtl.SelectedSerialNumber)
                settings.SetUInt32("IncrementText.TextStyle", self.labelStyleCtl.SelectedSerialNumber)
                settings.SetDouble("IncrementText.CurrentValue", currentValue)
                settings.SetDouble("IncrementText.IncrementValue", incValue)

                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

        if self.valueChangedByUser:
            (valid, r) = Double.TryParse(text)
            if valid and incValue:
                r = Math.Round(r / incValue) * incValue + incValue
                self.currentCtl.Value = r
            self.updateResult()
        else:
            self.currentCtl.Value += self.incrementCtl.Value
        ctrl.Clear()
        Keyboard.Focus(self.coordCtl)
        return