import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System.Windows.Controls import StackPanel
from System.IO import StreamReader
from System.Windows.Input import Keyboard
from System import Type, Tuple, Array, Double
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle

from Trimble.DiskIO import SearchPath, OptionsManager
from Trimble.Vce.Alignment.Linestring import Linestring,VerticalElementFactory, IVerticalNoCurve, IStationVerticalLocation
from Trimble.Vce.Alignment.Linestring import ElementFactory, IStraightSegment, IXYZLocation,ITangentArcSegment, ArcTangentType

from Trimble.Vce.Geometry import Point3D, Intersections
from Trimble.Vce.Geometry.PolySeg import PolySeg

from Trimble.Vce.Interfaces import Client
try:
    from Trimble.Sdk.UI import InputMethod
except:
    from Trimble.Sdk.Interfaces.UI import InputMethod

from Trimble.Vce.Core import GlobalSelection
from Trimble.Vce.UI.Controls.Wpf import CoordinateEdit, DistanceEdit

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "InsertLineSegment"
    cmdData.CommandName = "InsertLineSegment"
    cmdData.Caption = "_Insert Line Segment"
    cmdData.UIForm = "InsertLineSegment"
    cmdData.HelpFile = "Macros.chm"
    cmdData.HelpTopic = "22996"
    try:
        cmdData.Version = 1.62
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2018/10/17/tml-insertlinesegment"
    except:
        pass

    try:
        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "CAD"
        cmdData.ShortCaption = "Insert Segment"
        cmdData.ToolTipTitle = "Insert Line Segment"
        cmdData.ToolTipTextFormatted = "Inserts a straight line segment at defined location."

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

class InsertLineSegment(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\InsertLineSegment.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject
        self.macroName = "Insert Line Segment"
        self.tmlFileFolder = macroFileFolder

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Apply"
        buttons[1].Content = "Close"
        self.Caption = cmd.Command.Caption
        self.lineCtl.AddTypeFilter(clr.GetClrType(Linestring))
        OptionsManager.SetValue("InsertLine:Elev", self.elevCtl.AutoTabExtSkip)

    def lineCtl_ValueChanged(self, ctrl, e):
        if (self.lineCtl.SerialNumber):
            selectionSet = GlobalSelection.Items(self.currentProject)
            selectionSet.Set([self.lineCtl.SerialNumber])

    def coordCtl_ValueChanged(self, ctrl, e):
        # run cmd if change was due to mouse pick
        if e.Cause == InputMethod.Mouse:
            if self.elevCtl.AutoTabExtSkip:       
                self.OkClicked(None, None)
            else:
                self.elevCtl.Elevation = self.coordCtl.Coordinate.Z
                Keyboard.Focus(self.elevCtl)

    def elevCtl_ValueChanged(self, ctrl, e):
        if e.Cause == InputMethod.Mouse:     
            self.OkClicked(None, None)

    def CancelClicked(self, thisCmd, args):
        thisCmd.CloseUICommand()

    def OkClicked(self, thisCmd, e):
        line = self.lineCtl.Entity
        if not line:
            self.lineCtl.StatusMessage = "No line selected"
            return
        self.lineCtl.StatusMessage = ""
        coord = self.coordCtl.Coordinate
        if not coord.Is3D and not coord.Is2D :
            self.coordCtl.StatusMessage = "No coordinate defined"
            return
        self.coordCtl.StatusMessage = ""
        e = self.elevCtl.Elevation
        if not Double.IsNaN(e):
            coord.Z = e
            self.elevCtl.Clear() # clear elevation after use

        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)

        try:
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                poly = line.ComputePolySeg()
                if not poly or poly.NumberOfNodes < 2:
                    self.lineCtl.StatusMessage = "No line selected"
                    return
                begStation = poly.FirstNode.Station
                endStation = poly.LastNode.Station
                poly = poly.Clone() # make copy before extending
                poly.Extend(1000.0)
                rtn = poly.FindPointFromPoint(coord, None, None)
                if not rtn[0]:
                    return
                seg = rtn[2]
                if not seg:
                    return
                station = rtn[4]
                index = seg.Node.Index
                e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
                e.Position = coord
                if station > endStation:
                    line.AppendElement(e)
                elif station < begStation:
                    line.InsertElementAt(e, 0)
                else:
                    line.InsertElementAt(e, index)

                # you MUST commit the change to DB
                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

        if (self.lineCtl.AutoTabExtSkip):
            Keyboard.Focus(self.coordCtl)
        else:
            Keyboard.Focus(self.lineCtl)
        return