﻿"""
Macro to create Takeoff AOI objects from closed lines
"""
#region
import clr
# Reference the WPF assemblies
clr.AddReference('IronPython.Wpf')
import wpf
from System import String
from System.IO import StreamReader
from System.Windows.Controls import StackPanel
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap

from Trimble.Vce.Core.Components import WorldView, Project, UserSelection
from Trimble.Vce.Core import TransactMethodCall

from Trimble.Vce.Geometry import Point3D, Side

from Trimble.Vce.Interfaces.SnapIn import IPolyseg, IName
from Trimble.Vce.Interfaces.Client import CommandGranularity
from Trimble.Vce.Core import GlobalSelection

from Trimble.Vce.Gem import TakeoffSettings
from Trimble.Vce.ForeignCad import TextEntity, Leader
clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import SelectionContextMenuHandler
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents
#endregion
# define the command key and name
def Setup(cmdData, macroFileFolder):
    cmdData.Key = "LinesToAoi"
    cmdData.CommandName = "LinesToAOI"
    cmdData.Caption = "_Lines To AOI"
    cmdData.UIForm = "LinesToAoi"
    try:
        cmdData.Version = 1.53
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2019/01/16/tml-lines-to-aoi"
        cmdData.HelpFile = cmdData.MacroInfo
    except:
        pass

    try: 
        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "CAD"
        cmdData.ShortCaption = "Lines To AOI"
        cmdData.ToolTipTitle = "Create Takeoff AOIs from selected lines"
        cmdData.ToolTipTextFormatted = "Tool to create Takeoff AOI boundaries from a selection of lines."

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass 

class LinesToAoi(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\LinesToAoi.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject
        self.macroName = "Lines To Aoi"
        self.tmlFileFolder = macroFileFolder

    def OnLoad(self, cmd, buttons, event):
        self.Caption = cmd.Command.Caption
        self.objs.IsEntityValidCallback=self.IsValid
        optionMenu = SelectionContextMenuHandler()
        # remove options that don't apply here
        optionMenu.ExcludedCommands = "SelectObservations | SelectPoints | SelectDuplicatePoints"
        self.objs.ButtonContextMenu = optionMenu
        self.lType = clr.GetClrType(IPolyseg)
        self.iName = clr.GetClrType(IName)

    def CancelClicked(self, cmd, args):
        cmd.CloseUICommand()

    def IsValid(self, serial):
        o=self.currentProject.Concordance.Lookup(serial)
        if isinstance(o, self.lType):
            return True
        return False

    def OkClicked(self, cmd, e):
        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(CommandGranularity.Command, self.Caption)
        wv = self.currentProject[Project.FixedSerial.WorldView]
        try:
            numberFound = 0
            # the "with" statement will unroll any changes if something go wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                takeoffSettings = TakeoffSettings.ProvideObject(self.currentProject)
                aoiCollection = takeoffSettings.AOI_Collections
                SSCol = self.currentProject[Project.FixedSerial.StaticSelectionCollection]
                usType = clr.GetClrType(UserSelection)
                for o in self.objs.SelectedMembers(self.currentProject):
                    # check if object is "line" and can have a name
                    if isinstance(o, self.lType) and isinstance(o, self.iName):
                        polySeg = o.ComputePolySeg()
                        if polySeg:
                            if polySeg.IsClosed:
                                name = IName.Name.__get__(o)
                                if name:  
                                    # this closed line has a name. Check if AOI already exists
                                    numberFound += 1
                                    for s in SSCol:
                                        if isinstance(s, usType):
                                            if String.Compare(s.Name, name, True) == 0:
                                                if not s.Contains(o.SerialNumber):
                                                    # found AOI with same name. Add to that one
                                                    s.Add(o.SerialNumber)
                                                break
                                    else:
                                        # AOI not found
                                        myCollection = SSCol.Add(usType)
                                        myCollection.Name = name
                                        myCollection.Add(o.SerialNumber)
                                        # add new user selection to AOI list
                                        aoiCollection.Add(myCollection.SerialNumber)

                if not numberFound:
                    self.objs.StatusMessage = "Nothing Selected"
                    return

                # you MUST commit the change to DB or they will all be un-done (inside the "with" scope)
                failGuard.Commit()

        finally:
            self.currentProject.TransactionManager.AddEndMark(CommandGranularity.Command)

        cmd.CloseUICommand()

