﻿"""
TBC macro to create linestring from points sorted by station
"""
#region
import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System import Math, Action, Func, DateTime
from System.Text import Encoding
from System.Collections.Generic import List
from System.Windows import Window, MessageBox, MessageBoxResult, MessageBoxButton, Visibility
from System.Windows.Controls import StackPanel, Button, CheckBox
from System.Windows.Input import Keyboard, AccessKeyManager, Key
from System import Type, Tuple, Array, Double, String
from System.IO import StreamReader, MemoryStream, Path, File
from System.Globalization import CultureInfo
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys
from System.Windows.Forms import DialogResult

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle
from Trimble.Vce.SurveyCAD import ProjectLineStyle

from Trimble.DiskIO import SearchPath, OptionsManager

from Trimble.Vce.Alignment import ProfileView
from Trimble.Vce.Alignment.Linestring import Linestring,ElementFactory, IStraightSegment, IXYZLocation, IPointIdLocation

from Trimble.Vce.ForeignCad import TextEntity, MText, Text

from Trimble.Vce.Gem import Model3D

from Trimble.Vce.Coordinates import PointCollection, NEECoordinate, Point

from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings

from Trimble.Vce.Geometry import Point3D, Plane3D, Arc, Matrix4D, Vector3D, Side
from Trimble.Vce.Geometry.PolySeg import PolySeg
from Trimble.Vce.Geometry.PolySeg.Segment import Segment, Line

from Trimble.Vce.Interfaces.SnapIn import ISnapIn, IPolyseg, IHaveUcs, IMemberManagement, IName, IViewMember, TransformData, ResultsOfTransform
from Trimble.Vce.Interfaces.Point import IPoint, IPointLocation
from Trimble.Vce.Interfaces.Units import CoordinateOrder, LinearType, VerticalAngleType, AzimuthType, AngularType
from Trimble.Vce.Interfaces.FeatureCoding import IFeatureCodeHost, IFeatureCode
from Trimble.Vce.Interfaces.DescriptionFields import IDescriptionFields
from Trimble.Vce.Interfaces.Client import CommandGranularity
from Trimble.Vce.Interfaces import Client
from Trimble.Vce.Interfaces import ProgressBar

clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents
try:
    from Trimble.Sdk.UI import UIEventArgs
except:
    from Trimble.Sdk.Interfaces.UI import UIEventArgs
   
from Trimble.Vce.Core import GlobalSelection   
from Trimble.Vce.UI.Controls import VceMessageBox, SurfaceTypeLists

clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import SelectionContextMenuHandler
clr.AddReference("Trimble.Vce.UI.ConstructionCommands")
from Trimble.Vce.UI.ConstructionCommands import CreateHorizontalAlignmentCmd

#endregion

# define the command key and name
def Setup(cmdData, macroFileFolder):
    cmdData.Key = "LinestringFromPointsByStation"
    cmdData.CommandName = "LinestringFromPointsByStation"
    cmdData.Caption = "_Linestring From Points By Station"
    cmdData.UIForm = "LinestringFromPointsByStation"

    try:
        cmdData.Version = 0.51
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r""

        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "CAD"
        cmdData.ShortCaption = "Line from Points/Station"
        cmdData.ToolTipTitle = "Line from Points/Station"
        cmdData.ToolTipTextFormatted = "Create linestring from points sorted by station."

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass 

class MySortObject(object):
    def __init__(self, sta, offset, point):
        self.sta = sta
        self.offset = offset
        self.point = point

    def __lt__(self, other):
        # if withen 1 meter of "other", use offset to sort
        if Math.Abs(self.sta - other.sta) > 1:
            if self.sta < other.sta:
                return True
            return False

        if Math.Abs(self.offset - other.offset) > Point3D.Tolerance:
            if self.offset < other.offset:
                return True
            return False
        # if station and offset the same, just use serial number
        return self.point < other.point

class LinestringFromPointsByStation(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\LinestringFromPointsByStation.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject
        self.macroName = "Linestring From Points By Station"
        self.tmlFileFolder = macroFileFolder        

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Apply"
        buttons[1].Content = "Close"

        self.Caption = cmd.Command.Caption 
        self.refAlignment.SetEntityType(clr.GetClrType(ProfileView), self.currentProject)
        self.refAlignment.NewCmdCallback = CreateHorizontalAlignmentCmd.RunThisCmd
        self.objs.IsEntityValidCallback=self.IsValid
        optionMenu = SelectionContextMenuHandler()
        self.pType = clr.GetClrType(IPointLocation)
        
        # remove options that don't apply here
        optionMenu.ExcludedCommands = "SelectObservations | SelectDuplicatePoints"
        self.objs.ButtonContextMenu = optionMenu

        self.maxSegmentCtl.Distance = OptionsManager.GetDouble("LinestringFromPointsByStation:maxSegmentCtl", Double.NaN)
        self.lunits = self.currentProject.Units.Linear
        self.lfp = self.lunits.Properties.Copy()
        self.lfp.AddSuffix = False

    def CancelClicked(self, cmd, args):
        cmd.CloseUICommand()

    def IsValid(self, serial):
        o=self.currentProject.Concordance.Lookup(serial)
        return self.IsValidObj(o)

    def IsValidObj(self, obj):
        # needs to be a point type
        if isinstance(obj, self.pType) and isinstance(obj, clr.GetClrType(IFeatureCodeHost)):
            return True
        return False

    def CreateLineWithPoints(self, wv, layerSerial, name, points):
        l = wv.Add(clr.GetClrType(Linestring))
        l.Layer = layerSerial
        l.Name = name
        for p in points:
            e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IPointIdLocation))
            e.LocationSerialNumber = p
            l.AppendElement(e)             

    def OkClicked(self, cmd, e):
        wv = self.currentProject[Project.FixedSerial.WorldView]
        self.pointCollection = PointCollection.ProvidePointCollection(self.currentProject)
        h = self.refAlignment.SelectedSerialNumber()
        if not h:
            self.refAlignment.StatusMessage = "Select reference alignment"
            Keyboard.Focus(self.refAlignment)
            return False
        self.refAlignment.StatusMessage = ""
        self.objs.StatusMessage = ""

        maxSegment = self.maxSegmentCtl.Distance
        if Double.IsNaN(maxSegment) or maxSegment <= 0.0:
            self.maxSegmentCtl.StatusMessage = "Enter valid max segment length"
            return False
        self.maxSegmentCtl.StatusMessage = ""

        self.hal = self.refAlignment.SelectedEntity()
        self.alignPolyseg = self.hal.ComputePolySeg().Clone()
        self.alignPolyseg.Extend(10000.0)

        UIEvents.RaiseBeforeDataProcessing(self, UIEventArgs())
        # pause the graphics so every change doesn't draw
        wv.PauseGraphicsCache(True)
        try:
            self.currentProject.TransactionManager.AddBeginMark(CommandGranularity.Command, self.Caption)
            # the "with" statement will unroll any changes if something go wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:   
                numIgnored = 0
                nPts = 0
                listOfFCodes = {}
                # build dict based on feature code
                for o in self.objs.SelectedMembers(self.currentProject):
                    if not self.IsValidObj(o):
                        numIgnored += 1
                        continue
                    p = o.Position
                    rtn = self.alignPolyseg.FindPointFromPoint(p, None, None)
                    if not rtn[0]:
                        numIgnored += 1
                        continue
                    station = rtn[4]
                    offset =  rtn[5]
                    # we are only getting the first FC
                    fCode = ""
                    for fc in IFeatureCodeHost.FeatureCodes.GetValue(o):
                        fCode = fc.Code
                        break
                    # we only want points with a feature code
                    if not fCode:
                        numIgnored += 1
                        continue
                    key = fCode.ToLower()
                    if key in listOfFCodes:
                        list = listOfFCodes[key]
                    else:
                        list = []
                    list.Add(MySortObject(station, offset, o.SerialNumber))
                    nPts += 1
                    listOfFCodes[key] = list

                for k, v in listOfFCodes.items():
                    v.sort()
                    points = []
                    lastSta = 1.0e10
                    for o in v:
                        if not points:
                            p = self.currentProject.Concordance.Lookup(o.point)
                            layerSerial = p.Layer
                            name = ""
                            for fc in IFeatureCodeHost.FeatureCodes.GetValue(p):
                                name = fc.Code
                                break
                        if o.sta - lastSta > maxSegment:
                            if len(points) > 1:
                                self.CreateLineWithPoints(wv, layerSerial, name, points)
                            points = []
                        lastSta = o.sta
                        points.Add(o.point)
                    if len(points) > 1:
                        self.CreateLineWithPoints(wv, layerSerial, name, points)
                            
                failGuard.Commit()
                if numIgnored:
                    self.objs.StatusMessage = str(numIgnored) + " points ignored as they have no feature code"

        finally:
            OptionsManager.SetValue("LinestringFromPointsByStation:maxSegmentCtl", self.maxSegmentCtl.Distance)
            self.currentProject.TransactionManager.AddEndMark(CommandGranularity.Command)
            UIEvents.RaiseAfterDataProcessing(self, UIEventArgs())
            wv.PauseGraphicsCache(False)
        Keyboard.Focus(self.objs)

