﻿"""
Sample TBC macro to compute perpendicular distance from 3D point to surface
"""
#region
import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System import Math, Action, Func
from System.Text import Encoding
from System.Collections.Generic import List
from System.Windows import Window, MessageBox, MessageBoxResult, MessageBoxButton, Visibility
from System.Windows.Controls import StackPanel, Button, CheckBox
from System.Windows.Input import Keyboard, AccessKeyManager, Key
from System import Type, Tuple, Array, Double, String
from System.IO import StreamReader, MemoryStream, Path, File
from System.Globalization import CultureInfo
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle
from Trimble.Vce.SurveyCAD import ProjectLineStyle

from Trimble.DiskIO import SearchPath, OptionsManager

from Trimble.Vce.Alignment import ProfileView
from Trimble.Vce.Alignment.Linestring import Linestring, ElementFactory, IStraightSegment, IXYZLocation, IPointIdLocation

from Trimble.Vce.ForeignCad import TextEntity, MText, Text
from Trimble.Vce.ForeignCad import Point as CadPoint

from Trimble.Vce.Gem import Model3D

from Trimble.Vce.Data import MediaFolder

from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings

from Trimble.Vce.Coordinates import PointCollection, StationOffsetPoint, NEECoordinate, Point

from Trimble.Vce.Geometry import Point3D, Plane3D, Arc, Matrix4D, Vector3D, Side, Line2D, Line3D, Triangle2D
from Trimble.Vce.Geometry.PolySeg import PolySeg
from Trimble.Vce.Geometry.PolySeg.Segment import Segment

from Trimble.Vce.Interfaces.SnapIn import ISnapIn, IPolyseg, IHaveUcs, IMemberManagement, IName, IViewMember
from Trimble.Vce.Interfaces.Point import IPoint
from Trimble.Vce.Interfaces.Units import CoordinateOrder, LinearType, VerticalAngleType, AzimuthType, AngularType
from Trimble.Vce.Interfaces.FeatureCoding import IFeatureCodeHost, IFeatureCode
from Trimble.Vce.Interfaces.DescriptionFields import IDescriptionFields
from Trimble.Vce.Interfaces.Client import CommandGranularity
from Trimble.Vce.Interfaces import Client
from Trimble.Vce.Interfaces import ProgressBar

clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents
clr.AddReference("Trimble.Vce.UI.Controls")
try:
    from Trimble.Sdk.UI import UIEventArgs
except:
    from Trimble.Sdk.Interfaces.UI import UIEventArgs
from Trimble.Vce.Core import GlobalSelection
from Trimble.Vce.UI.Controls import VceMessageBox, SurfaceTypeLists
from Trimble.Vce.UI.Interfaces import ICommand

clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import SelectionContextMenuHandler
clr.AddReference("Trimble.Vce.UI.ConstructionCommands")
from Trimble.Vce.UI.ConstructionCommands import CreateHorizontalAlignmentCmd

#endregion

# define the command key and name
def Setup(cmdData, macroFileFolder):
    cmdData.Key = "PerpendicularDistanceToSurface"
    cmdData.CommandName = "PerpendicularDistanceToSurface"
    cmdData.Caption = "_Perpendicular Distance To Surface"
    cmdData.UIForm = "PerpendicularDistanceToSurface"

    try:
        cmdData.Version = 1.02
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r""

        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "Surface"
        cmdData.ShortCaption = "Distance To Surface"
        cmdData.ToolTipTitle = "Perpendicular Distance To Surface"
        cmdData.ToolTipTextFormatted = "Compute point projected onto specified surface."

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass 

class PerpendicularDistanceToSurface(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\PerpendicularDistanceToSurface.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject
        self.macroFileFolder = macroFileFolder
        self.macroName = "Perpendicular Distance To Surface"

    def OnLoad(self, cmd, buttons, event):
        self.Caption = cmd.Command.Caption
        types = SurfaceTypeLists.AllWithCorridorStrataSurfacesAndTunnel
        self.refSurface.FilterByEntityTypes = types   
        self.objs.IsEntityValidCallback=self.IsValid
        self.pType = clr.GetClrType(IPoint)
        self.cadPoint = clr.GetClrType(CadPoint)
        self.lunits = self.currentProject.Units.Linear
        # we don't want the units to be included
        self.lfp = self.lunits.Properties.Copy()
        self.lfp.AddSuffix = False
        self.pointCollection = PointCollection.ProvidePointCollection(self.currentProject)
        
        # remove options that don't apply here
        optionMenu = SelectionContextMenuHandler()
        optionMenu.ExcludedCommands = "SelectObservations | SelectDuplicatePoints"
        self.objs.ButtonContextMenu = optionMenu

        self.incPts.IsChecked = OptionsManager.GetBool("PerpendicularDistanceToSurface:incPts", True)
        self.incLines.IsChecked = OptionsManager.GetBool("PerpendicularDistanceToSurface:incLines", False)
        self.incFile.IsChecked = OptionsManager.GetBool("PerpendicularDistanceToSurface:incFile", False)
        try:
            import InfragisticsDlls
            from InfragisticsDlls import ButtonTool, ToolBase
            b = ButtonTool("MyDeleteButton")
            cp = TrimbleOffice.TheOffice.CommandProcessor
            cmdTool = cp.CommandList["delete"]
            b.SharedProps.Caption = cmdTool.CleanCaption()
            b.SharedProps.AppearancesSmall.Appearance.Image = cmdTool.ImageSmall
            cmd.Command.InternalTools = Array[ToolBase]([b])
        except:
            pass

    def ToolClick(self, env, p):
        if p.Tool.Key == "MyDeleteButton":
            TrimbleOffice.TheOffice.CommandProcessor.ExecuteCommand("delete", None);

    def CancelClicked(self, cmd, args):
        cmd.CloseUICommand()

    def IsValid(self, serial):
        o=self.currentProject.Concordance.Lookup(serial)
        return self.IsValidObj(o)

    def IsValidObj(self, obj):
        # needs to be a 3D point type
        if isinstance(obj, self.pType):
            p = obj.Position
            if p.Is3D:
                return True
        if isinstance(obj, self.cadPoint):
            p = obj.Position
            if p.Is3D:
                return True
        return False

    def ConvertToDisplay(self, value):
        # this will convert the number to the current display units (all values are stored as meters)
        return self.lunits.Format(value, self.lfp)

    def ComputePerpendicularToCrossSection(self, surface, polyHal, pt):
        seg = clr.Reference[Segment]()
        t = clr.Reference[float]()
        staRef = clr.Reference[float]()
        offsetRef = clr.Reference[float]()
        pcl = clr.Reference[Point3D]()
        v = clr.Reference[Vector3D]()
        side = clr.Reference[Side]()
        # first get points station value along alignment
        rtn = polyHal.FindPointFromPoint(pt, seg, t, pcl, staRef, v, offsetRef, side)
        if not rtn:
            return None
        # then compute cross-section at this station
        poly = surface.ComputeCrossSection(None, polyHal, staRef.Value, 200.0, 200.0, False, False, 0)
        if not poly:
            return None
        offset = offsetRef.Value
        if side.Value == Side.Left:
            offset = -offset
        # Find shortest distance to cross-section
        (found, pnt, sta)=poly.FindPointFromPoint(Point3D(offset, pt.Z))
        if not found:
            return None
        # convert from offset,elev to XYZ
        v = v.Value
        v.Normalize()
        pNew = pcl.Value + pnt.X * v
        pNew.Z = pnt.Y
        return pNew

    def ComputePerpendicularLocationToSurface(self, surface, polyHal, pt):
        if polyHal != None:
            return self.ComputePerpendicularToCrossSection(surface, polyHal, pt)
        nTri = surface.NumberOfTriangles
        bestPt = Point3D()
        bestDistance = 1.0e10
        for t in range(nTri):
            if not surface.IsTriangleMaterialPresent(t):
                continue
            ptPlane = self.NearPointToTriangle(surface, t, pt)
            #self.CreateLine(self.wv, Project.FixedSerial.LayerZero, pt, ptPlane)
            d = (pt - ptPlane).Length2
            if d < bestDistance:
                bestDistance = d
                bestPt = ptPlane
        return bestPt

    def NearPointToTriangle(self, surface, iTri, pt):
        (v,p0) = surface.GetTriangleVertex(iTri, 0)
        (v,p1) = surface.GetTriangleVertex(iTri, 1)
        (v,p2) = surface.GetTriangleVertex(iTri, 2)
        (plane, p0x, p1x, p2x) = Plane3D(p0, p1, p2)
        ptPlane = Plane3D.IntersectWithRayPerpendicular(plane, pt)
        (inside, p0x,p1x,p2x,ptx) = Triangle2D.IsPointInside(p0, p1, p2, ptPlane)
        if inside:
            return ptPlane
        (d0, p0x) = self.Distance2ToLine(p0, p1, pt)
        (d1, p1x) = self.Distance2ToLine(p1, p2, pt)
        (d2, p2x) = self.Distance2ToLine(p2, p0, pt)
        if d0 < d1:
            if d0 < d2:
                return p0x
        if d1 < d2:
            return p1x
        return p2x
        
    def Distance2ToLine(self, p0, p1, pt):
        (p0x,p1x,ptx,t,d2) = Line3D.Distance2ToPoint(p0, p1, pt)
        if t < 0:
             return ((pt - p0).Length2, p0)
        if t > 1.0:
            return ((pt - p1).Length2, p1)
        (p1x, p2x, ptx) = Line3D.InterpolatePoint(p0, p1, t)
        return ((pt - ptx).Length2, ptx)

    def CreateLine(self, wv, layerSerial, p1, p2):
        l = wv.Add(clr.GetClrType(Linestring))
        e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
        e.Position = p1 
        l.AppendElement(e)
        e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
        e.Position = p2
        l.AppendElement(e)       
        l.Layer = layerSerial

    def OkClicked(self, cmd, e):
        wv = self.currentProject[Project.FixedSerial.WorldView]
        surface = self.refSurface.SelectedEntity()
        if surface == None:
            return
        polyHal = None
        if surface.AlignmentSerialNumber != 0:
            wv = self.currentProject[Project.FixedSerial.WorldView]
            alignment = wv.Lookup(surface.AlignmentSerialNumber)
            if alignment != None:
                polyHal = alignment.ComputePolySeg()

        count = self.objs.Count
        if self.incFile.IsChecked:
            if self.WriteFile(count, surface, polyHal):
                return
        if self.incLines.IsChecked:
            self.CreatePointsOrLines(wv, True, count, surface, polyHal)
        if self.incPts.IsChecked:
            self.CreatePointsOrLines(wv, False, count, surface, polyHal)

        ProgressBar.TBC_ProgressBar.Title = ""
        OptionsManager.SetValue("PerpendicularDistanceToSurface:incPts", self.incPts.IsChecked)
        OptionsManager.SetValue("PerpendicularDistanceToSurface:incLines",  self.incLines.IsChecked)
        OptionsManager.SetValue("PerpendicularDistanceToSurface:incFile",  self.incFile.IsChecked )

        cmd.CloseUICommand()

    def WriteFile(self, count, surface, polyHal):        
        # points are stored in meters, get conversion factor
        self.unitFactor = self.currentProject.Units.Linear.Convert(1.0, LinearType.Display)
        self.cultureInfo = CultureInfo.CurrentCulture.TextInfo.ListSeparator
        self.coordsInNE = self.currentProject.Units.Coordinate.CoordinateOrder == CoordinateOrder.NEZ
        fileName = self.fileNameCtl.Text
        if not fileName:
            # empty name, need to report name error
            self.refSurface.StatusMessage = "Enter valid file name"
            Keyboard.Focus(self.fileNameCtl)
            return True
        if fileName.IndexOfAny(Path.GetInvalidPathChars()) != -1:
            self.refSurface.StatusMessage = "Invalid char in file name"
            Keyboard.Focus(self.fileNameCtl)
            return True
        if not Path.IsPathRooted(fileName):
            fileName = Path.Combine(SearchPath.ExportAppDataPath,fileName)

        fileName = Path.ChangeExtension(fileName, ".csv")
        if File.Exists(fileName):
            File.Delete(fileName)

        ProgressBar.TBC_ProgressBar.Title = "Writing file..."
        with open(fileName, 'w') as f:   
            self.file = f
            sComma = str(self.cultureInfo) # could be period in some regions
            # write header info
            sCsv = "Point ID" + sComma
            if self.coordsInNE:
                sCsv += "Pt North" + sComma + "Pt East" + sComma + "Pt Elevation" + sComma + "Distance" + sComma + \
                   "North" + sComma + "East" + sComma + "Elevation" + "\n"
            else:
                sCsv += "Pt East" + sComma + "Pt North" + sComma + "Pt Elevation" + sComma + "Distance" + sComma + \
                   "East" + sComma + "North" + sComma + "Elevation" + "\n"
            f.write(sCsv)

            i = 0
            for o in self.objs.SelectedMembers(self.currentProject):
                percent = i * 100 // count
                i += 1
                if ProgressBar.TBC_ProgressBar.SetProgress(percent):
                    return True
                if not self.IsValidObj(o):
                    continue
                p = o.Position
                name = ""
                if isinstance(o, clr.GetClrType(IName)):
                    name = o.Name
                # make sure name doesn't have " character
                if name.IndexOf(sComma, 0):
                    name = name.Replace(sComma, "_")
                if self.coordsInNE:
                    sCsv = name + sComma + self.ConvertToDisplay(p.Y) + sComma + self.ConvertToDisplay(p.X)
                else:
                    sCsv = name + sComma + self.ConvertToDisplay(p.X) + sComma + self.ConvertToDisplay(p.Y)

                ptOnSurface = self.ComputePerpendicularLocationToSurface(surface, polyHal, p)
                if ptOnSurface != None:
                    dist = (ptOnSurface - p).Length
                    sCsv = sCsv + sComma + self.ConvertToDisplay(p.Z) + sComma + self.ConvertToDisplay(dist)
                    if self.coordsInNE:
                        sCsv = sCsv + sComma + self.ConvertToDisplay(ptOnSurface.Y) + sComma + self.ConvertToDisplay(ptOnSurface.X)
                    else:
                        sCsv = sCsv + sComma + self.ConvertToDisplay(ptOnSurface.X) + sComma + self.ConvertToDisplay(ptOnSurface.Y)
                    sCsv = sCsv + sComma + self.ConvertToDisplay(ptOnSurface.Z)
                sCsv = sCsv +  "\n"
                self.file.write(sCsv)
        return False

    def CreatePointsOrLines(self, wv, createLines, count, surface, polyHal):        
        UIEvents.RaiseBeforeDataProcessing(self, UIEventArgs())
        # pause the graphics so every change doesn't draw
        wv.PauseGraphicsCache(True)
        try:
            layerSerial = self.LineLayers.SelectedSerialNumber
            self.currentProject.TransactionManager.AddBeginMark(CommandGranularity.Command, self.Caption)
            surface = self.refSurface.SelectedEntity()
            if createLines:
                ProgressBar.TBC_ProgressBar.Title = "Creating lines..."
            else:
                ProgressBar.TBC_ProgressBar.Title = "Creating points..."
            # the "with" statement will unroll any changes if something go wrong
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                i = 0
                for o in self.objs.SelectedMembers(self.currentProject):
                    percent = i * 100 // count
                    i += 1
                    if ProgressBar.TBC_ProgressBar.SetProgress(percent):
                        return
                    if not self.IsValidObj(o):
                        continue
                    p = o.Position
                    ptOnSurface = self.ComputePerpendicularLocationToSurface(surface, polyHal, p)
                    if ptOnSurface == None:
                        continue # computed station off alignment
                    if createLines:
                        self.CreateLine(wv, layerSerial, p, ptOnSurface)
                    else:
                        pnt = self.pointCollection.Add(clr.GetClrType(Point))
                        pnt.AddCoordinate(NEECoordinate(ptOnSurface))
                        pnt.Layer = layerSerial

                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(CommandGranularity.Command)
            UIEvents.RaiseAfterDataProcessing(self, UIEventArgs())
            wv.PauseGraphicsCache(False)



