﻿"""
Report on corridor template nodes
"""
#region Reference stuff
import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System import Math, Action, Func, Type, UInt32
from System.Text import Encoding
from System.Collections.Generic import List
from System.Windows import Window, MessageBox, MessageBoxResult, MessageBoxButton, Visibility
from System.Windows.Controls import StackPanel, Button, CheckBox
from System.Windows.Input import Keyboard, AccessKeyManager, Key
from System import Type, Tuple, Array, Double, String
from System.IO import StreamReader, MemoryStream, Path, File
from System.Globalization import CultureInfo
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap, Color
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle
from Trimble.Vce.SurveyCAD import ProjectLineStyle

from Trimble.DiskIO import OptionsManager

from Trimble.Vce.Alignment import ProfileView
from Trimble.Vce.Alignment.Linestring import Linestring
from Trimble.Vce.Corridor import Corridor, LevelContainer, Level, AvgEndAreaEarthworkEngine, IManifestedTemplate, RoadCodeType

from Trimble.Vce.ForeignCad import TextEntity, MText, Text

from Trimble.Vce.Gem import Model3D

from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings

from Trimble.Vce.Geometry import Point3D, Plane3D, Arc, Matrix4D, Vector3D, Side
from Trimble.Vce.Geometry.PolySeg import PolySeg

from Trimble.Vce.Interfaces.SnapIn import ISnapIn, IPolyseg, IHaveUcs, IMemberManagement, IName
from Trimble.Vce.Interfaces.Point import IPoint, IPointLocation
from Trimble.Vce.Interfaces.Units import CoordinateOrder, LinearType, VerticalAngleType, AzimuthType, AngularType
from Trimble.Vce.Interfaces.Client import CommandGranularity
from Trimble.Vce.Interfaces import Client, ProgressBar
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents
from Trimble.Vce.Core import GlobalSelection      
from Trimble.Vce.UI.Controls import VceMessageBox

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "PtsToCorridor"
    cmdData.CommandName = "PtsToCorridor"
    cmdData.Caption = "_Pts To Corridor"
    cmdData.UIForm = "PtsToCorridor"

    # if you are going to include a help file, define file and topic id here
    cmdData.HelpFile = "MyMacros.chm"
    try:
        cmdData.Version = 1.0
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r""
    except:
        pass   

    try:
        # if you want the macro included in the ribbon, fill in the following 5
        # lines
        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "Corridor"
        cmdData.ShortCaption = "Cut-Sheet"
        cmdData.ToolTipTitle = "Pts To Corridor"
        cmdData.ToolTipTextFormatted = "Compare points to location on corridor."

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

# the name of this class must match name from cmdData.UIForm (above)
class PtsToCorridor(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\PtsToCorridor.xaml") as s:
            wpf.LoadComponent(self, s)

        self.currentProject = currentProject
        # get units
        self.lunits = self.currentProject.Units.Linear
        # we don't want the units to be included
        self.lfp = self.lunits.Properties.Copy()
        self.lfp.AddSuffix = False
        self.staUnits = self.currentProject.Units.Station
        self.pType = clr.GetClrType(IPointLocation)

        self.corridor.SetEntityType(clr.GetClrType(Corridor), currentProject)
        levelContainer = LevelContainer.ProvideLevelContainer(currentProject)
        defaultLevelId = levelContainer.DefaultSerialNumber
        self.materialLayer.SearchContainer = levelContainer.SerialNumber
        self.materialLayer.UseSelectionEngine = False
        self.materialLayer.SetEntityType(clr.GetClrType(Level), currentProject)
        levelName = OptionsManager.GetString("PtsToCorridor:levelName", "")
        # try to match name of level
        for l in levelContainer:
            if String.Compare(levelName, l.Name, True) == 0:
                self.materialLayer.SelectBySerialNumber(l.SerialNumber)
                break

        self.stationIntervalCtl.Distance = OptionsManager.GetDouble("PtsToCorridor:stationIntervalCtl",
            self.lunits.Convert(LinearType.Display, 10.0, LinearType.Meter))
        self.stationHal.IsChecked = OptionsManager.GetBool("PtsToCorridor:stationHal", False)
        self.stationVal.IsChecked = OptionsManager.GetBool("PtsToCorridor:stationVal", False)
        self.stationTemplates.IsChecked = OptionsManager.GetBool("PtsToCorridor:stationTemplates", False)
            
        self.gridSpacingCtl.Distance = OptionsManager.GetDouble("PointsToGrid:Grid", Double.NaN)

        optionMenu = SelectionContextMenuHandler()
        # remove options that don't apply here
        optionMenu.ExcludedCommands = "SelectObservations | SelectDuplicatePoints"
        self.objs.ButtonContextMenu = optionMenu

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Compute"
        buttons[1].Content = "Close"
        self.Caption = cmd.Command.Caption

    def CancelClicked(self, sender, e):
        sender.CloseUICommand()

    def IsValid(self, serial):
        o=self.currentProject.Concordance.Lookup(serial)
        return self.IsValidObj(o)

    def GridPoint(self, point, staList, iSta):
        if iSta in staList:
            pts = staList[iSta]
        else:
            pts = []
        pts.append(point.SerialNumber)
        staList[iSta] = pts

    def Round(self, val, grid):
        if val > 0:
            v =  int((val + grid/2) / grid)
        else:
            v  = int((val - grid/2) / grid)
        return v

    def IsValidObj(self, obj):
        # needs to be a point type
        if isinstance(obj, self.pType):
            # should be 3D point
            if Double.IsNaN(obj.Position.Z):
                return False
            return True
        return False

    def OkClicked(self, sender, e):
        cor = self.corridor.SelectedEntity()
        if not cor:
            self.corridor.StatusMessage = "Select corridor"
            Keyboard.Focus(self.corridor)
            return False
        self.corridor.StatusMessage = ""

        gridSize = self.gridSpacingCtl.Distance
        if Double.IsNaN(gridSize) or gridSize <= 0.0:
            self.gridSpacingCtl.StatusMessage = "Enter valid scale factor"
            return False
        self.gridSpacingCtl.StatusMessage = ""

        self.materialSerial = self.materialLayer.SelectedEntity().SerialNumber
        self.ignoreUndefined = self.ignoreUndefinedRoadCodes.IsChecked
        self.doStation = self.reportStaHeader.IsChecked
        self.doLabels = self.reportLabels.IsChecked
        hal = cor.AlignmentSerialNumber
        hal = self.currentProject.Concordance.Lookup(hal)
        self.gridSize = gridSize
        self.alignPolyseg = self.hal.ComputePolySeg().Clone()
        self.alignPolyseg.Extend(10000.0)
        stationTable = hal.HorizontalAlignment.StationTable
        # compute cells
        along = {}
        for o in self.objs.SelectedMembers(self.currentProject):
            if not self.IsValidObj(o):
                continue
            p = o.Position

            rtn = self.alignPolyseg.FindPointFromPoint(p, None, None)
            if not rtn[0]:
                continue
            station = rtn[4]
            IStation  = stationTable.DistanceToStation(station);
            self.GridPoint(o, along, self.Round(station, gridSize), self.Round(offset, gridSize))
        self.CreateReportForFieldLayout(stnList, hal, cor)
            
        OptionsManager.SetValue("PtsToCorridor:reportLayout", self.reportLayout.SelectedIndex)
        OptionsManager.SetValue("PtsToCorridor:reportDirection", self.reportDirection.SelectedIndex)
        OptionsManager.SetValue("PtsToCorridor:stationIntervalCtl", self.stationIntervalCtl.Distance)
        OptionsManager.SetValue("PtsToCorridor:stationHal", self.stationHal.IsChecked)
        OptionsManager.SetValue("PtsToCorridor:stationVal", self.stationVal.IsChecked)
        OptionsManager.SetValue("PtsToCorridor:stationTemplates", self.stationTemplates.IsChecked)

        self.currentProject.TransactionManager.AddEndMark(CommandGranularity.Command)

        sender.CloseUICommand()
    
    def CreateReportForFieldLayout(self, stnList, alignment, corridor):
        ProgressBar.TBC_ProgressBar.Title = "Generating Report"

        row = 1
        count = stnList.Count
        i = 0
        for s in stnList:
            station = s.Station
            percent = i * 100 // count
            i += 1
            if ProgressBar.TBC_ProgressBar.SetProgress(percent):
                break
            manifestedTemplate = corridor.GetManifestedRawData(station).ManifestedTemplate
            segments = IManifestedTemplate.Segments.__get__(manifestedTemplate)
            if segments: 
                staString = self.staUnits.Format(station, alignment)
                if self.doStation:
                    txt = "Station " + staString
                    sdt = s.StationDataTypes



                nodes = self.BuildNodeList(manifestedTemplate, segments)
                if self.reportDirection.SelectedIndex == 0:
                    col = 1
                    if self.doLabels:
                        self.AcrossReportHeaders(eReport, row)
                        col += 1
                    row = self.AcrossReport(eReport, row, col, nodes)
                else:
                    if self.doLabels:
                        self.DownReportHeaders(eReport, row)
                        row += 1
                    row = self.DownReport(eReport, row, staString, nodes)
            row += 1

    def AcrossReport(self, eReport, rowStart, col, nodes): 
        row = rowStart
        for avgOffset, node, crd, offset, deltaElev, slope in sorted(nodes.values(), key=self.getSortKey):        
            pass

        return row

    def BuildNodeList(self, manifestedTemplate, segments):
        nodes = {}
        for segment in segments:
            if not segment.IsMemberOfLevel(self.materialSerial):
                continue
            if self.ignoreUndefined:
                if segment.Node2.VirtualNode.RoadCode == RoadCodeType.Undefined:
                    continue
            avgOffset = (segment.Node2.Offset + segment.Node1.Offset) / 2.0
            offset = segment.Node2.Offset - segment.Node1.Offset
            deltaElev = segment.Node2.Elevation - segment.Node1.Elevation
            if offset:
                slope = Math.Atan2(deltaElev, Math.Abs(offset))
            else:
                slope = Double.NaN
            id = segment.Node2.VirtualNode.ID
            if not nodes.ContainsKey(id):
                c = IManifestedTemplate.ToWorldView(manifestedTemplate, segment.Node2.Coordinate)
                nodes.Add(id, (avgOffset, segment.Node2, c, offset, deltaElev, slope))
            id = segment.Node1.VirtualNode.ID
            if not nodes.ContainsKey(id):
                c = IManifestedTemplate.ToWorldView(manifestedTemplate, segment.Node1.Coordinate)
                nodes.Add(id, (segment.Node1.Offset, segment.Node1, c, offset, deltaElev, Double.NaN))
        return nodes
