﻿"""
Sample TBC macro to select points based of delta elevation to surface
"""
#region
import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System import Math, Action, Func
from System.Text import Encoding
from System.Collections.Generic import List
from System.Windows import Window, MessageBox, MessageBoxResult, MessageBoxButton, Visibility
from System.Windows.Controls import StackPanel, Button, CheckBox
from System.Windows.Input import Keyboard, AccessKeyManager, Key
from System import Type, Tuple, Array, Double, String
from System.IO import StreamReader, MemoryStream, Path, File
from System.Globalization import CultureInfo
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle
from Trimble.Vce.SurveyCAD import ProjectLineStyle

from Trimble.DiskIO import SearchPath, OptionsManager

from Trimble.Vce.Alignment import ProfileView
from Trimble.Vce.Alignment.Linestring import Linestring

from Trimble.Vce.ForeignCad import TextEntity, MText, Text

from Trimble.Vce.Gem import Model3D

from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings

from Trimble.Vce.Geometry import Point3D, Plane3D, Arc, Matrix4D, Vector3D, Side
from Trimble.Vce.Geometry.PolySeg import PolySeg

from Trimble.Vce.Interfaces.SnapIn import ISnapIn, IPolyseg, IHaveUcs, IMemberManagement, IName, IViewMember
from Trimble.Vce.Interfaces.Point import IPoint, IPointLocation
from Trimble.Vce.Interfaces.Units import CoordinateOrder, LinearType, VerticalAngleType, AzimuthType, AngularType
from Trimble.Vce.Interfaces.FeatureCoding import IFeatureCodeHost, IFeatureCode
from Trimble.Vce.Interfaces.DescriptionFields import IDescriptionFields
from Trimble.Vce.Interfaces.Client import CommandGranularity
from Trimble.Vce.Interfaces import Client
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents
from Trimble.Vce.Core import GlobalSelection    
from Trimble.Vce.UI.Controls import VceMessageBox, SurfaceTypeLists

clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import SelectionContextMenuHandler
clr.AddReference("Trimble.Vce.UI.ConstructionCommands")
from Trimble.Vce.UI.ConstructionCommands import CreateHorizontalAlignmentCmd
#endregion

# define the command key and name
def Setup(cmdData, macroFileFolder):
    cmdData.Key = "SelectByDeltaElevation"
    cmdData.CommandName = "SelectByDeltaElevation"
    cmdData.Caption = "_Select By Delta Elevation"
    cmdData.UIForm = "SelectByDeltaElevation"
    cmdData.HelpFile="Macros.chm"
    cmdData.HelpTopic="xxxx"

    try:
        cmdData.Version = 1.01
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r""

        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "CAD"
        cmdData.ShortCaption = "Select By Delta El"
        cmdData.ToolTipTitle = "Select By Delta Elevation"
        cmdData.ToolTipTextFormatted = "Selects points based on delta elevation to reference surface."

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass 

class SelectByDeltaElevation(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\SelectByDeltaElevation.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject

    def OnLoad(self, cmd, buttons, event):
        self.Caption = cmd.Command.Caption
        types = Array[Type](SurfaceTypeLists.AllWithCutFillMap)
        self.refSurface.FilterByEntityTypes = types   
        self.objs.IsEntityValidCallback=self.IsValid
        optionMenu = SelectionContextMenuHandler()
        # remove options that don't apply here
        optionMenu.ExcludedCommands = "SelectObservations | SelectDuplicatePoints"
        self.objs.ButtonContextMenu = optionMenu
        self.pType = clr.GetClrType(IPointLocation)
        self.toleranceCtl.Distance = OptionsManager.GetDouble("SelectByDeltaElevation:Tolerance", Double.NaN)
        self.selectIn.IsChecked = OptionsManager.GetBool("SelectByDeltaElevation:selectIn", False)
        self.selectOut.IsChecked = OptionsManager.GetBool("SelectByDeltaElevation:selectOut", True)

    def CancelClicked(self, cmd, args):
        cmd.CloseUICommand()

    def IsValid(self, serial):
        o=self.currentProject.Concordance.Lookup(serial)
        return self.IsValidObj(o)

    def IsValidObj(self, obj):
        # needs to be a point type
        if isinstance(obj, self.pType):
            return True
        return False

    def Dispose(self, thisCmd, disposing):
        OptionsManager.SetValue("SelectByDeltaElevation:Tolerance", self.toleranceCtl.Distance)
        OptionsManager.SetValue("SelectByDeltaElevation:selectIn", self.selectIn.IsChecked)
        OptionsManager.SetValue("SelectByDeltaElevation:selectOut", self.selectOut.IsChecked)

    def OkClicked(self, cmd, e):
        self.currentProject.TransactionManager.AddBeginMark(CommandGranularity.Command, self.Caption)
        wv = self.currentProject[Project.FixedSerial.WorldView]

        if self.refSurface.SelectedSerial == 0:
            self.refSurface.StatusMessage = "Enter a surface"
            return False
        self.refSurfaceModel = self.refSurface.SelectedEntity()
        tolerance = self.toleranceCtl.Distance
        if Double.IsNaN(tolerance):
            self.toleranceCtl.StatusMessage = "Enter tolerance"
        keepOutPoints = self.selectOut.IsChecked
        newSelection = []
        for o in self.objs.SelectedMembers(self.currentProject):
            if not self.IsValidObj(o):
                continue
            p = o.Position
            if  Double.IsNaN(p.Z):
                continue

            found,pSurface = self.refSurfaceModel.PickSurface(p)# get point from surface
            if found:
                if abs(pSurface.Z - p.Z) > tolerance:
                    # outside tolerance
                    if keepOutPoints:
                        newSelection.Add(o.SerialNumber)
                else:
                    #inside tolerance
                    if not keepOutPoints:
                        newSelection.Add(o.SerialNumber)
  
        # now update global selection
        if newSelection:
            GlobalSelection.Items(self.currentProject).Set(newSelection)

        cmd.CloseUICommand()

