﻿"""
TBC Macro to compute legend of site materials used in a surface
"""
#region
# Reference the WPF assemblies
import clr
clr.AddReference('IronPython.Wpf')
import wpf

from Trimble.Vce.Core.Components import WorldView, Project, Layer
from Trimble.Vce.Core.Components.TextStyle import TextHeightMode, TextJustification
from Trimble.Vce.Core import TransactMethodCall
from System import Type, Tuple, Array, Double, Math, String
from System.Globalization import CultureInfo
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import DialogResult
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
from System.IO import StreamReader, Path, File
from System.Collections.Generic import List
from System.Windows import  Window, ResourceDictionary, Visibility
from System.Windows.Controls import StackPanel
from System.Windows.Input import Keyboard

from Trimble.Vce.Interfaces.Units import LinearType
from Trimble.Vce.Interfaces.SnapIn import IPolyseg, ISpacialContainer, ISnapInHelpers, IViewSite, ISnapIn
from Trimble.Vce.Interfaces.Client import CommandGranularity
from Trimble.Vce.Interfaces import ProgressBar
from Trimble.Vce.Interfaces.Construction import IMaterialProperties, ISiteImprovementAreaMaterial

from Trimble.DiskIO import SearchPath, OptionsManager

from Trimble.Vce.Gem import Model3D, DtmIsopachs, Model3DCompSettings, TakeoffSiteImprovement, TakeoffLayerMaterials, SurfaceTexture

from Trimble.Vce.Alignment import ProfileView
from Trimble.Vce.Alignment.Linestring import Linestring,ElementFactory, IStraightSegment, IXYZLocation
from Trimble.Vce.Alignment.Linestring import Polygon
from Trimble.Vce.Macros import GenericContainer

from Trimble.Vce.Geometry import Point3D, Line2D, Side, Vector3D
from Trimble.Vce.Geometry.PolySeg import PolySeg
from Trimble.Vce.Geometry.PolySeg.Segment import Segment, Line
from Trimble.Vce.Geometry.Region import Region, RegionBuilder

from Trimble.Vce.Data.Construction.Materials import MiscArea
clr.AddReference("Trimble.Vce.UI.Construction")
from Trimble.Vce.UI.Construction import ThicknessUtilities

from Trimble.Vce.ForeignCad import PolyLine, MText, EntityBase, AttachmentPoint
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents
clr.AddReference("Trimble.Vce.UI.Controls")
try:
    from Trimble.Sdk.UI import UIEventArgs  
except:
    from Trimble.Sdk.Interfaces.UI import UIEventArgs
from Trimble.Vce.Core import GlobalSelection    
from Trimble.Vce.UI.Controls import SurfaceTypeLists, VceMessageBox

from Trimble.Vce.UI.UIManager import TrimbleOffice

clr.AddReference("Trimble.Vce.UI.ConstructionCommands")
from Trimble.Vce.UI.ConstructionCommands import CreateHorizontalAlignmentCmd
clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import ViewHelper

#endregion
def Setup(cmdData, macroFileFolder):
    cmdData.Key = "SiteImprovementLegend"
    cmdData.CommandName = "SiteImprovementLegend"
    cmdData.Caption = "_Site Improvement Legend"
    cmdData.UIForm = "SiteImprovementLegend"
    try:
        cmdData.Version = 1.53
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2019/01/18/tml-site-improvement-legend"
        cmdData.HelpFile = cmdData.MacroInfo
    except:
        pass

    try:
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

class SiteImprovementLegend(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\SiteImprovementLegend.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject
        self.macroName = "Site Improvement Legend"
        self.tmlFileFolder = macroFileFolder 

        # we are goint to set the suffix for inch display to be "
        self.inch = self.currentProject.Units.Linear.Find(LinearType.Inch)
        self.CurrentSuffixIndex = self.inch.SuffixIndex

    def Dispose(self, thisCmd, disposing):
        self.inch.SuffixIndex = self.CurrentSuffixIndex

    def GetContainerIfEditing(self):
        items = GlobalSelection.SelectedMembers(self.currentProject)
        for sn in items:
            parent = sn.GetSite()
            if isinstance(parent, clr.GetClrType(GenericContainer)):
                return parent
            return None # only check first object in list
        return None

    def OnLoad(self, cmd, buttons, event):
        self.Caption = cmd.Command.Caption
        types = Array[Type](SurfaceTypeLists.AllWithCutFillMap)
        self.surface.FilterByEntityTypes = types  
        self.locationCtl.ValueChanged += self.locationCtlChanged
        self.lunits = self.currentProject.Units.Linear
        self.lfp = self.lunits.Properties.Copy()
        self.lfp.AddSpace = False # no space between number and suffix
        if self.currentProject.Units.Linear.DisplayType != LinearType.Meter:
            self.lfp.NumberOfDecimals = 2
        else:
            self.lfp.NumberOfDecimals = 0
        self.myContainer = self.GetContainerIfEditing()
        self.HostingView = None
        if self.myContainer:
            # delete old data
            list = self.myContainer.List[clr.GetClrType(ISnapIn)]()
            newSelection = []
            for o in list:
                newSelection.Add(o.SerialNumber)
            GlobalSelection.Items(self.currentProject).Set(newSelection)
            self.columnWidthCtl.Distance = self.myContainer.GetDouble("SiteImprovementLegend:columnWidthCtl", Double.NaN)
            self.colorBoxSizeCtl.Distance = self.myContainer.GetDouble("SiteImprovementLegend:colorBoxSizeCtl", Double.NaN)
            self.isDetailedCtl.IsChecked = self.myContainer.GetBoolean("SiteImprovementLegend:isDetailedCtl", False)
            self.surface.SelectBySerialNumber(self.myContainer.GetUInt32("SiteImprovementLegend:surface"))
            self.layerCtl.SelectedSerialNumber = self.myContainer.GetUInt32("SiteImprovementLegend:layerCtl")
            self.numberCols.Value = self.myContainer.GetInt32("SiteImprovementLegend:numberCols", 1)

            self.titleCtl.Text = self.myContainer.GetString("SiteImprovementLegend:titleCtl")
            self.titleStyleCtl.SelectedSerialNumber = self.myContainer.GetUInt32("SiteImprovementLegend:titleStyleCtl")
            self.siTitleCtl.Text = self.myContainer.GetString("SiteImprovementLegend:siTitleCtl")
            self.siTitleStyleCtl.SelectedSerialNumber = self.myContainer.GetUInt32("SiteImprovementLegend:siTitleStyleCtl")
            self.siStyleCtl.SelectedSerialNumber = self.myContainer.GetUInt32("SiteImprovementLegend:siStyleCtl")
            self.siColorTitleCtl.Text = self.myContainer.GetString("SiteImprovementLegend:siColorTitleCtl")
        else:
            concordance = self.currentProject.Concordance
            layerCollection = concordance[Project.FixedSerial.LayerContainer]
            textCollection = concordance[Project.FixedSerial.TextStyleContainer]
            self.columnWidthCtl.Distance = OptionsManager.GetDouble("SiteImprovementLegend:columnWidthCtl", Double.NaN)
            self.colorBoxSizeCtl.Distance = OptionsManager.GetDouble("SiteImprovementLegend:colorBoxSizeCtl", Double.NaN)
            self.isDetailedCtl.IsChecked = OptionsManager.GetBool("SiteImprovementLegend:isDetailedCtl", False)
            l = OptionsManager.GetString("SiteImprovementLegend:layerCtl", "")
            layer = layerCollection.GetLayer(l)
            if layer:
                self.layerCtl.SelectedSerialNumber = layer.SerialNumber
            self.numberCols.Value = OptionsManager.GetInt("SiteImprovementLegend:numberCols", 1)

            self.titleCtl.Text = OptionsManager.GetString("SiteImprovementLegend:titleCtl", "")
            ts = OptionsManager.GetString("SiteImprovementLegend:titleStyleCtl", "")
            tStyle = textCollection.GetTextStyle(ts)
            if tStyle:
                self.titleStyleCtl.SelectedSerialNumber = tStyle.SerialNumber
            self.siTitleCtl.Text = OptionsManager.GetString("SiteImprovementLegend:siTitleCtl", "")
            self.titleCtl.Text = OptionsManager.GetString("SiteImprovementLegend:titleCtl", "")
            ts = OptionsManager.GetString("SiteImprovementLegend:siTitleStyleCtl", "")
            tStyle = textCollection.GetTextStyle(ts)
            if tStyle:
                self.siTitleStyleCtl.SelectedSerialNumber = tStyle.SerialNumber
            ts = OptionsManager.GetString("SiteImprovementLegend:siStyleCtl", "")
            tStyle = textCollection.GetTextStyle(ts)
            if tStyle:
                self.siStyleCtl.SelectedSerialNumber = tStyle.SerialNumber
            self.siColorTitleCtl.Text = OptionsManager.GetString("SiteImprovementLegend:siColorTitleCtl", "")

        self.isDetailedCtl.Checked += self.isDetailedChanged
        self.isDetailedCtl.Unchecked += self.isDetailedChanged
        self.isDetailedChanged(None, None)

    def isDetailedChanged(self, sender, args):
        if self.isDetailedCtl.IsChecked:
            self.summaryControls.Visibility = Visibility.Collapsed
            self.summaryControls2.Visibility = Visibility.Collapsed
            self.detailControls.Visibility = Visibility.Visible
        else:
            self.detailControls.Visibility = Visibility.Collapsed
            self.summaryControls.Visibility = Visibility.Visible
            self.summaryControls2.Visibility = Visibility.Visible

    def locationCtlChanged(self, ctrl, e):
        # we lock the distance controls to correct view so plan/sheet units are correct
        container = ViewHelper.GetContainerForView(self.locationCtl.ClickWindow)
        if isinstance(container, clr.GetClrType(ISpacialContainer)):
            csd = container.CoordinateSystemDefinition
            self.HostingView = container
        else:
            # sheet view has a tree structure of containers (plan set, sheet set, sheet)
            # so we need to get controlling container
            csd = ISnapInHelpers.GetCoordinateSystemDefinition(container)
            sn,self.HostingView = ISnapInHelpers.GetViewSite(container)

        self.columnWidthCtl.LockedCoordinateSystem = csd
        self.columnWidthCtl.SynchronizeTextBox()
        self.colorBoxSizeCtl.LockedCoordinateSystem = csd
        self.colorBoxSizeCtl.SynchronizeTextBox()

    def CancelClicked(self, sender, args):
        sender.CloseUICommand()

    def ComputeTextHeight(self, textStyle, viewContainer, worldPlotScale):
        height = textStyle.Height
        if isinstance(viewContainer, clr.GetClrType(IViewSite)):
            plotScale = viewContainer.PlotScale
        else:
            csd,view = ISnapInHelpers.GetViewSite(viewContainer)
            plotScale = view.PlotScale
        
        if plotScale < 1.1:
            # sheet view
            if textStyle.HeightMode == TextHeightMode.Ground:
                height = height / worldPlotScale
            else:
                height /= 1000.0
        else:
            if textStyle.HeightMode == TextHeightMode.Paper:
                height = height * plotScale / 1000
        return height

    def GetMaterialsLevelsFromSurface(self, surface):
        concordance = self.currentProject.Concordance
        layerCollection = concordance[Project.FixedSerial.LayerContainer]
        gem = surface.Gem
        nMaterials = gem.NumberOfMaterials
        SIs = {}
        for i in range(nMaterials):
            snMaterial,mmap,count = gem.GetMaterial(i)
            if snMaterial and count:
                m = concordance.Lookup(snMaterial)
                if m:
                    newColor = None
                    name = None
                    mLayers = None
                    if isinstance(m, clr.GetClrType(TakeoffSiteImprovement)):
                        mat = concordance.Lookup(m.MaterialProperties)                       
                        if isinstance(mat, clr.GetClrType(MiscArea)):
                            snMaterial = mat.SerialNumber
                            name = mat.Name
                            newColor = mat.Color
                            mLayers = mat.MaterialLayers
                        else:
                            continue
                    elif isinstance(m, clr.GetClrType(TakeoffLayerMaterials)):
                        sim = m.SiteImprovement
                        if isinstance(sim, clr.GetClrType(MiscArea)):
                            snMaterial = sim.SerialNumber
                            newColor = sim.Color
                            name = sim.Name
                            mLayers = sim.MaterialLayers

                    elif isinstance(m, clr.GetClrType(MiscArea)):
                        snMaterial = m.SerialNumber
                        newColor = m.Color
                        name = m.Name
                        mLayers = m.MaterialLayers

                    if name and not snMaterial in SIs:
                        SIs.Add(snMaterial, (name, newColor, mLayers))
        return SIs

    def MakeDetailSI(self, x, y, sizeOfBox, viewContainer, layer, color):
        poly = viewContainer.Add(clr.GetClrType(Polygon))
        poly.Layer = layer
        poly.Color = color
        poly.FillColor = color
        poly.FillTransparency = self.TransparencyPercentage
        e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
        e.Position = Point3D(x, y)
        poly.AppendElement(e) 
        e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
        e.Position = Point3D(x + sizeOfBox, y)
        poly.AppendElement(e) 
        e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
        e.Position = Point3D(x + sizeOfBox, y - sizeOfBox)
        poly.AppendElement(e) 
        e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
        e.Position = Point3D(x, y - sizeOfBox)
        poly.AppendElement(e) 

    def CreateSI_LevelLegend(self, viewContainer, worldPlotScale, layer, numCols, pOrigin, title, titleStyle, 
            siTextStyle, siStyle,  widthName, widthColor,  SIs):
        x = pOrigin.X
        y = pOrigin.Y
        tol = widthColor / 4.0
        x2 = x + numCols * (widthName + widthColor) + 2 * tol
        self.AddLine(viewContainer, layer, x, y, x2, y)  
        
        if title:
            h = self.ComputeTextHeight(titleStyle, viewContainer, worldPlotScale)
            textObj = viewContainer.Add(clr.GetClrType(MText))
            textObj.TextString = title
            textObj.Layer = layer
            textObj.TextStyleSerial = titleStyle.SerialNumber
            textObj.WidthFactor = titleStyle.WidthFactor
            textObj.AddWhiteOut = titleStyle.WhiteOut
            textObj.ObliqueAngle = titleStyle.ObliqueAngle
            textObj.Height = h
            h *= 1.6
            p = Point3D((x + x2) / 2, y - h / 2)
            textObj.Point0 = p
            textObj.AttachPoint = AttachmentPoint.MiddleCenter
            y -= h
            self.AddLine(viewContainer, layer, x, y, x2, y)
        y = y - tol
 
        h1 = self.ComputeTextHeight(siTextStyle, viewContainer, worldPlotScale)
        h2 = self.ComputeTextHeight(siStyle, viewContainer, worldPlotScale)

        col = -1
        rowH = 0
        for name,color,mLayers in sorted(SIs.values(), key=self.getSortKey): 
            col += 1
            if col >= numCols:
                col = 0
                y = y - max(1.2 * widthColor, rowH + h2 / 2)
                rowH = 0
            xc = x + tol + col * (widthName + widthColor)
            self.MakeDetailSI(xc, y, widthColor, viewContainer, layer, color)

            textObj = viewContainer.Add(clr.GetClrType(MText))
            textObj.TextString = name
            textObj.Layer = layer
            textObj.TextStyleSerial = siTextStyle.SerialNumber
            textObj.WidthFactor = siTextStyle.WidthFactor
            textObj.AddWhiteOut = siTextStyle.WhiteOut
            textObj.ObliqueAngle = siTextStyle.ObliqueAngle
            textObj.Height = h1
            xT = xc + h1 / 2 + widthColor
            p = Point3D(xT, y)
            textObj.Point0 = p
            textObj.AttachPoint = AttachmentPoint.TopLeft

            yT = y - 1.6 * h1
            for mat in mLayers:
                if not mat:
                    continue
                d = self.lunits.Format(LinearType.Meter, mat.Depth, self.lfp, LinearType.DisplaySmall)
                textObj = viewContainer.Add(clr.GetClrType(MText))
                textObj.TextString = d + " " + mat.Name
                textObj.Layer = layer
                textObj.TextStyleSerial = siStyle.SerialNumber
                textObj.WidthFactor = siStyle.WidthFactor
                textObj.AddWhiteOut = siStyle.WhiteOut
                textObj.ObliqueAngle = siStyle.ObliqueAngle
                textObj.Height = h2
                p = Point3D(xT, yT)
                yT = yT - 1.6 * h2
                textObj.Point0 = p
                textObj.AttachPoint = AttachmentPoint.TopLeft
            r = 1.6 * h1 + len(mLayers) * (1.6 * h2)
            if r > rowH:
                rowH = r

        y = y - max(1.2 * widthColor, rowH)
        self.AddLine(viewContainer, layer, x2, pOrigin.Y, x2, y)
        self.AddLine(viewContainer, layer, x2, y, pOrigin.X, y)
        self.AddLine(viewContainer, layer, pOrigin.X, y, pOrigin.X, pOrigin.Y)
        
        return

    def GetMaterialsFromSurface(self, surface):
        concordance = self.currentProject.Concordance
        layerCollection = concordance[Project.FixedSerial.LayerContainer]
        gem = surface.Gem
        nMaterials = gem.NumberOfMaterials
        SIs = {}
        for i in range(nMaterials):
            snMaterial,mmap,count = gem.GetMaterial(i)
            if snMaterial and count:
                m = concordance.Lookup(snMaterial)
                if m:
                    newColor = None
                    name = None
                    if isinstance(m, clr.GetClrType(TakeoffSiteImprovement)):
                        mat = concordance.Lookup(m.MaterialProperties)
                        name = mat.Name
                        snMaterial = mat.SerialNumber
                        if isinstance(mat, clr.GetClrType(ISiteImprovementAreaMaterial)):
                            c = mat.Color
                        else:
                            c = m.Color
                            if EntityBase.isColorByLayer(c) or EntityBase.isColorByBlock(c):
                                # user layer property to get color
                                layer = layerCollection.Lookup(m.Layer)
                                if (layer):
                                    c = layer.DefaultColor
                                else:
                                    c = System.Drawing.Color.Green
                        newColor = c
                    elif isinstance(m, clr.GetClrType(TakeoffLayerMaterials)):
                        sim = m.SiteImprovement
                        if (sim):
                            snMaterial = sim.SerialNumber
                            newColor = sim.Color
                            name = sim.Name
                    elif isinstance(m, clr.GetClrType(SurfaceTexture)):
                        #we don't pass the serial number of SurfaceTexture any
                        #more.  Only the material                        
                        c = m.Color
                        if EntityBase.isColorByLayer(c) or EntityBase.isColorByBlock(c):
                            # user layer property to get color
                            layer = layerCollection.Lookup(m.Layer)
                            if (layer):
                                c = layer.DefaultColor
                            else:
                                c = System.Drawing.Color.Green
                        newColor = c
                        snMaterial = m.SerialNumber
                        name = m.Name
                    elif isinstance(m, clr.GetClrType(ISiteImprovementAreaMaterial)):
                        newColor = m.Color
                        snMaterial = m.SerialNumber
                        name = m.Name
                    elif isinstance(m, clr.GetClrType(IMaterialProperties)):
                        newColor = m.Color
                        snMaterial = m.SerialNumber
                        name = m.Name

                    if name:
                        SIs.Add(snMaterial, (name, newColor))
        return SIs

    def AddLine(self, viewContainer, layer, x1, y1, x2, y2):
        polyline = viewContainer.Add(clr.GetClrType(PolyLine))
        polyline.Layer = layer
        n = PolyLine.PolyLineNode()
        n.X = x1
        n.Y = y1
        polyline.AddNode(n)
        n.X = x2
        n.Y = y2
        polyline.AddNode(n)

    def getSortKey(self, items):
        return items[0] # sort on name

    def CreateSI_Legend(self, viewContainer, worldPlotScale, layer, pOrigin, title, titleStyle, titleSI,siTextStyle, titleColor,
                    siStyle,  widthName, widthColor,  SIs):
        x = pOrigin.X
        y = pOrigin.Y
        x2 = x + widthName + widthColor
        self.AddLine(viewContainer, layer, x, y, x2, y)
       
        if title:
            h = self.ComputeTextHeight(titleStyle, viewContainer, worldPlotScale)
            textObj = viewContainer.Add(clr.GetClrType(MText))
            textObj.TextString = title
            textObj.Layer = layer
            textObj.TextStyleSerial = titleStyle.SerialNumber
            textObj.WidthFactor = titleStyle.WidthFactor
            textObj.AddWhiteOut = titleStyle.WhiteOut
            textObj.ObliqueAngle = titleStyle.ObliqueAngle
            textObj.Height = h
            h *= 1.6
            p = Point3D(x + (widthName + widthColor) / 2, y - h / 2)
            textObj.Point0 = p
            textObj.AttachPoint = AttachmentPoint.MiddleCenter
            y -= h
            self.AddLine(viewContainer, layer, x, y, x2, y)

        y2 = y # save location of top of ISs
        if titleSI or titleColor:
            h = self.ComputeTextHeight(siTextStyle, viewContainer, worldPlotScale)
            textObj = viewContainer.Add(clr.GetClrType(MText))
            if not titleSI:
                titleSI = "Material"
            textObj.TextString = titleSI
            textObj.Layer = layer
            textObj.TextStyleSerial = siTextStyle.SerialNumber
            textObj.WidthFactor = siTextStyle.WidthFactor
            textObj.AddWhiteOut = siTextStyle.WhiteOut
            textObj.ObliqueAngle = siTextStyle.ObliqueAngle
            textObj.Height = h
            h2 = h * 1.6
            p = Point3D(x + h2 / 2, y - h2 / 2)
            textObj.Point0 = p
            textObj.AttachPoint = AttachmentPoint.MiddleLeft

            textObj = viewContainer.Add(clr.GetClrType(MText))
            if not titleColor:
                titleColor = "Color"
            textObj.TextString = titleColor
            textObj.Layer = layer
            textObj.TextStyleSerial = siTextStyle.SerialNumber
            textObj.WidthFactor = siTextStyle.WidthFactor
            textObj.AddWhiteOut = siTextStyle.WhiteOut
            textObj.ObliqueAngle = siTextStyle.ObliqueAngle
            textObj.Height = h
            p = Point3D(x + widthName + h2 / 2, y - h2 / 2)
            textObj.Point0 = p
            textObj.AttachPoint = AttachmentPoint.MiddleLeft
            y -= h2
            self.AddLine(viewContainer, layer, x, y, x2, y)
 
        h = self.ComputeTextHeight(siStyle, viewContainer, worldPlotScale)
        for name,color in sorted(SIs.values(), key=self.getSortKey):        
            textObj = viewContainer.Add(clr.GetClrType(MText))
            textObj.TextString = name
            textObj.Layer = layer
            textObj.TextStyleSerial = siStyle.SerialNumber
            textObj.WidthFactor = siStyle.WidthFactor
            textObj.AddWhiteOut = siStyle.WhiteOut
            textObj.ObliqueAngle = siStyle.ObliqueAngle
            textObj.Height = h
            h2 = h * 1.6
            p = Point3D(x + h2 / 2, y - h2 / 2)
            textObj.Point0 = p
            textObj.AttachPoint = AttachmentPoint.MiddleLeft
            poly = viewContainer.Add(clr.GetClrType(Polygon))
            poly.Layer = layer
            poly.Color = color
            poly.FillColor = color
            poly.FillTransparency = 0
            offset = h2 / 8
            e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
            e.Position = Point3D(x + widthName + offset, y - offset)
            poly.AppendElement(e) 
            e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
            e.Position = Point3D(x2 - offset, y - offset)
            poly.AppendElement(e) 
            e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
            e.Position = Point3D(x2 - offset, y + offset - h2)
            poly.AppendElement(e) 
            e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
            e.Position = Point3D(x + widthName + offset, y + offset - h2)
            poly.AppendElement(e) 
            y -= h2
            self.AddLine(viewContainer, layer, x, y, x2, y)

        self.AddLine(viewContainer, layer, pOrigin.X, pOrigin.Y, pOrigin.X, y)
        x = x + widthName
        self.AddLine(viewContainer, layer, x, y2, x, y)
        self.AddLine(viewContainer, layer, x2, pOrigin.Y, x2, y)
        return

    def OkClicked(self, sender, e):
        version = TrimbleOffice.TheOffice.ProductInfo.ProductVersion
        if version.Major == 5 and version.Build < 6950:
            self.surface.StatusMessage = "This macro requires version 5.1 or later!!!"
            return False
        if self.surface.SelectedSerial == 0:
            self.surface.StatusMessage = "Enter a surface"
            return False

        pOrigin = self.locationCtl.Coordinate
        if pOrigin.IsUndefined :
            self.locationCtl.StatusMessage = "No coordinate defined"
            return False
        self.locationCtl.StatusMessage = ""

        if self.isDetailedCtl.IsChecked:
            tmp = self.numberCols.Value
            if self.numberCols.StatusMessage:
                return False

        container = ViewHelper.GetContainerForView(self.locationCtl.ClickWindow) # usually a sheet but could be WorldView

        colWidth = self.columnWidthCtl.Distance
        if Double.IsNaN(colWidth) or colWidth <= 0.0:
            self.columnWidthCtl.StatusMessage = "Enter valid column width"
            return False
        self.columnWidthCtl.StatusMessage = ""

        boxWidth = self.colorBoxSizeCtl.Distance
        if Double.IsNaN(boxWidth) or boxWidth <= 0.0:
            self.colorBoxSizeCtl.StatusMessage = "Enter valid box size"
            return False
        self.colorBoxSizeCtl.StatusMessage = ""

        try:
            self.currentProject.TransactionManager.AddBeginMark(CommandGranularity.Command, self.Caption)
            UIEvents.RaiseBeforeDataProcessing(self, UIEventArgs())
            # pause the graphics so every change doesn't draw
            self.HostingView.PauseGraphicsCache(True)
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                wv = self.currentProject[Project.FixedSerial.WorldView]
                surfaceModel = self.surface.SelectedEntity()
                if self.myContainer:
                    # check if legend is placed in same container (plan vs sheet)
                    if not container.Lookup(self.myContainer.SerialNumber):
                        self.myContainer.GetSite().Remove(self.myContainer.SerialNumber)
                        self.myContainer = None

                if self.myContainer:
                    # delete lines/text from previous run
                    list = self.myContainer.List[clr.GetClrType(ISnapIn)]()
                    for o in list:
                        self.myContainer.Remove(o.SerialNumber)
                else:
                    self.myContainer = container.Add(clr.GetClrType(GenericContainer))

                self.TransparencyPercentage = surfaceModel.TransparencyPercentage
                # set the suffix used by inch formatter to " (we need to reset
                # this in the dispose call)
                self.inch.SuffixIndex = 2
                if self.isDetailedCtl.IsChecked:
                    SIs = self.GetMaterialsLevelsFromSurface(surfaceModel)
                    self.CreateSI_LevelLegend(self.myContainer, wv.PlotScale, self.layerCtl.SelectedSerialNumber, int(self.numberCols.Value), pOrigin,
                        self.titleCtl.Text, self.titleStyleCtl.SelectedSnapIn, self.siTitleStyleCtl.SelectedSnapIn,
                        self.siStyleCtl.SelectedSnapIn, self.columnWidthCtl.Distance, self.colorBoxSizeCtl.Distance, SIs)
                    pass
                else:
                    SIs = self.GetMaterialsFromSurface(surfaceModel)
                    self.CreateSI_Legend(self.myContainer, wv.PlotScale, self.layerCtl.SelectedSerialNumber, pOrigin,
                        self.titleCtl.Text, self.titleStyleCtl.SelectedSnapIn , self.siTitleCtl.Text, self.siTitleStyleCtl.SelectedSnapIn,
                        self.siColorTitleCtl.Text, self.siStyleCtl.SelectedSnapIn, self.columnWidthCtl.Distance, self.colorBoxSizeCtl.Distance, SIs)

                # we are going to save the settings to two locations.  One tied
                # to this legend. The other in a shared location for use when creating new
                # legends.
                self.myContainer.SetDouble("SiteImprovementLegend:columnWidthCtl", self.columnWidthCtl.Distance)
                OptionsManager.SetValue("SiteImprovementLegend:columnWidthCtl", self.columnWidthCtl.Distance)
                self.myContainer.SetDouble("SiteImprovementLegend:colorBoxSizeCtl", self.colorBoxSizeCtl.Distance)
                OptionsManager.SetValue("SiteImprovementLegend:colorBoxSizeCtl", self.colorBoxSizeCtl.Distance)
                self.myContainer.SetBoolean("SiteImprovementLegend:isDetailedCtl",  self.isDetailedCtl.IsChecked)
                OptionsManager.SetValue("SiteImprovementLegend:isDetailedCtl",  self.isDetailedCtl.IsChecked)
                if Double.IsNaN(self.numberCols.Value):
                    self.myContainer.SetInt32("SiteImprovementLegend:numberCols", 1)
                else:
                    self.myContainer.SetInt32("SiteImprovementLegend:numberCols", int(self.numberCols.Value))
                    OptionsManager.SetValue("SiteImprovementLegend:numberCols", int(self.numberCols.Value))                  

                self.myContainer.SetUInt32("SiteImprovementLegend:layerCtl",  self.layerCtl.SelectedSerialNumber)
                layer = self.layerCtl.SelectedSnapIn
                if layer:
                    OptionsManager.SetValue("SiteImprovementLegend:layerCtl", layer.Name)

                self.myContainer.SetString("SiteImprovementLegend:titleCtl",  self.titleCtl.Text)
                OptionsManager.SetValue("SiteImprovementLegend:titleCtl", self.titleCtl.Text)
                self.myContainer.SetUInt32("SiteImprovementLegend:titleStyleCtl",  self.titleStyleCtl.SelectedSerialNumber)
                tStyle = self.titleStyleCtl.SelectedSnapIn
                if tStyle:
                    OptionsManager.SetValue("SiteImprovementLegend:titleStyleCtl", tStyle.Name)

                self.myContainer.SetString("SiteImprovementLegend:siTitleCtl",  self.siTitleCtl.Text)
                OptionsManager.SetValue("SiteImprovementLegend:siTitleCtl", self.siTitleCtl.Text)
                self.myContainer.SetUInt32("SiteImprovementLegend:siTitleStyleCtl",  self.siTitleStyleCtl.SelectedSerialNumber)
                tStyle = self.siTitleStyleCtl.SelectedSnapIn
                if tStyle:
                    OptionsManager.SetValue("SiteImprovementLegend:siTitleStyleCtl", tStyle.Name)

                self.myContainer.SetString("SiteImprovementLegend:siColorTitleCtl",  self.siColorTitleCtl.Text)
                OptionsManager.SetValue("SiteImprovementLegend:siColorTitleCtl", self.siColorTitleCtl.Text)
                self.myContainer.SetUInt32("SiteImprovementLegend:siStyleCtl",  self.siStyleCtl.SelectedSerialNumber)
                tStyle = self.siStyleCtl.SelectedSnapIn
                if tStyle:
                    OptionsManager.SetValue("SiteImprovementLegend:siStyleCtl", tStyle.Name)

                self.myContainer.SetUInt32("SiteImprovementLegend:surface",  self.surface.SelectedSerialNumber())
                
                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(CommandGranularity.Command)
            UIEvents.RaiseAfterDataProcessing(self, UIEventArgs())
            self.HostingView.PauseGraphicsCache(False)

        sender.CloseUICommand()