﻿"""
Macro to list all the objects found in the current project
"""
# Reference the WPF assemblies
import clr
clr.AddReference('IronPython.Wpf')
import wpf
clr.AddReference("PresentationFramework")
clr.AddReference("PresentationCore")
clr.AddReference("windowsbase")
from System.Windows import  Window, MessageBox, MessageBoxResult, MessageBoxButton
from System.Windows.Controls import TreeView, TreeViewItem
from System import Environment, StringComparison
from System.IO import StreamReader
from System.Text import StringBuilder
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap

from Trimble.Vce.Core.Components import SnapInAttributeExtension, Project
from Trimble.Vce.Interfaces.SnapIn import IMemberManagement, IName, ISnapIn
from Trimble.Vce.Interfaces.Client import CommandGranularity
try:
    from Trimble.Sdk.Interfaces import IFeatureManager
except:
    from Trimble.Vce.Interfaces.Feature import IFeatureManager
from Trimble.Vce.Interfaces.FeatureCoding import IFeatureCodeManager
from Trimble.Vce.Interfaces.FeatureDefinitions import IFeatureCodeDefinitionContainer, IFeatureDefinitionBlock

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "ViewProjectData"
    cmdData.CommandName = "ViewProjectData"
    cmdData.Caption = "_View Project Data"
    cmdData.HelpFile="Macros.chm"
    cmdData.HelpTopic="22204"
    try:
        cmdData.Version = 1.53
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2018/10/22/tml-view-project-data"

        cmdData.DefaultTabKey =  "" # leave off menu for now "Macros"
        cmdData.DefaultTabGroupKey = "General"
        cmdData.ShortCaption = "Project Data"
        cmdData.ToolTipTitle = "View Project Data"
        cmdData.ToolTipTextFormatted = "Diagnostic to examine all objects contained in this project."

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

def Execute(cmd, currentProject, macroFileFolder, parameters): 
    form = ViewProjectDataDialog(currentProject, macroFileFolder).ShowDialog()

class ViewProjectDataDialog(Window): # this inherits from the WPF window control
    def __init__(self, currentProject, macroFileFolder):

        with StreamReader(macroFileFolder + r"\ViewProjectData.xaml") as s:
            wpf.LoadComponent(self, s) 

        self.myProject = currentProject
        self.CloseBtn.Click += self.Button_Click_Cancel
        self.Items = []

        self.Tree.SelectedItemChanged += self.Selection_Changed
        tvi = TreeViewItem()
        tvi.Tag = currentProject
        s = currentProject.GetType()
        tvi.Header = s.ToString()
        self.Tree.Items.Add(tvi)

    def updateBtn_Click(self, sender, e):
        self.mask = self.searchMask.Text
        tvi = self.Tree.Items[0]
        tvi.Items.Clear()
        if self.mask.StartsWith("="):
            serial = self.mask.Substring(1)
            try:
                serial = int(serial)
                o = self.myProject.Concordance.Lookup(serial)
                tvi1 = TreeViewItem() 
                tvi1.Tag = o
                s = o.GetType()
                text = s.ToString() + " [" + o.SerialNumber.ToString() + "]"
                tvi1.Header = text
                tvi.Items.Add(tvi1)
                tvi.IsExpanded = True
                return
            except:
                pass
        self.AddSnapInToTree(tvi, self.myProject, clr.GetClrType(IMemberManagement))
        tvi.IsExpanded = True

    def AddSnapInToTree(self, containerTreeNode,container, collectionInterface):
        include = False
        for o in container:
            tvi = TreeViewItem() 
            tvi.Tag = o
            s = o.GetType()
            text = s.ToString() + " [" + o.SerialNumber.ToString() + "]"
            if self.mask:
                valid =  text.IndexOf(self.mask, StringComparison.CurrentCultureIgnoreCase) >= 0
            else:
                valid = True
            if isinstance(o, collectionInterface):
                valid = self.AddSnapInToTree(tvi, o, collectionInterface) or valid
            elif isinstance(o, clr.GetClrType(IFeatureManager)) or \
                    isinstance(o, clr.GetClrType(IFeatureDefinitionBlock)) or \
                    isinstance(o, clr.GetClrType(IFeatureCodeDefinitionContainer)) or \
                    isinstance(o, clr.GetClrType(IFeatureCodeManager)):                
                valid = self.AddSnapInToTree(tvi, o, collectionInterface) or valid
            if valid:
                include = True
                tvi.Header = text
                containerTreeNode.Items.Add(tvi)
        return include

    def Button_Click_Cancel(self, sender, e):
        self.Close()
        pass

    def Selection_Changed(self, sender, e):
        o = e.NewValue.Tag
        hasControl = Control.ModifierKeys & Keys.Control
        if hasControl:
            rtn = MessageBox.Show("Delete Selected Object", "OK to DELETE this object", MessageBoxButton.OKCancel)
            if rtn == MessageBoxResult.OK:
                # we should delete this object
                o.GetSite().Remove(o.SerialNumber)
                e.NewValue.Parent.Items.Remove(e.NewValue)
                self.myProject.TransactionManager.AddEndMark(CommandGranularity.Command)
                return

        self.OutputEnityInfo(o)

    def MyFilterCriteria(self, filter, filterCriteria):
        return True

    def BuildInterfaceListString(self, type):
        interfaces = type.FindInterfaces(self.MyFilterCriteria, type)
        list = ""
        for i in interfaces:
            list += " " + i.ToString() + Environment.NewLine
        return list

    def BuildInheritenceListString(self, type, includeThis):
        if type == clr.GetClrType(object):
            return ""
        list = ""
        if includeThis:
            list = " " + type.ToString() + Environment.NewLine
        return list + self.BuildInheritenceListString(type.BaseType, True)

    def OutputEnityInfo(self, selectedObject):
        sb = StringBuilder()
        # Output type
        t = selectedObject.GetType()
        tString = t.ToString()
        infoString = "Type: " + tString
        sb.Append(infoString + Environment.NewLine + Environment.NewLine)

        if isinstance(selectedObject, clr.GetClrType(IName)):
            # obj.Name was not working for some objects so just call interface
            sb.Append("Name: " + IName.Name.__get__(selectedObject) + Environment.NewLine)

        # Output inheritance tree
        tString = self.BuildInheritenceListString(t, False)
        if tString:
            sb.Append("Inheritance:" + Environment.NewLine + tString + Environment.NewLine)

        # Output interfaces implemented
        tString = self.BuildInterfaceListString(t)
        if tString:
            sb.Append("Interfaces:" + Environment.NewLine + tString + Environment.NewLine)

        # Print out the things looking at us
        observers = self.myProject.Concordance.GetObserversOf(selectedObject.SerialNumber)
        sb.Append("Observers:" + Environment.NewLine)
        for o in observers:
            sb.Append(" " + o.GetType().ToString() + " " + o.SerialNumber.ToString() + Environment.NewLine)

        sb.Append(Environment.NewLine)

        #Print out the things we are looking at.
        observes = self.myProject.Concordance.GetIsObservedBy(selectedObject.SerialNumber)
        sb.Append("Observing:" + Environment.NewLine)
        for o in observes:
            sb.Append(" " + o.GetType().ToString() + " " + o.SerialNumber.ToString() + Environment.NewLine)

        # the project object does not have user attributes
        if selectedObject.SerialNumber != Project.FixedSerial.Project:
            userAtributes = SnapInAttributeExtension.UserAttributes.Overloads[ISnapIn](selectedObject)
            if userAtributes.Count > 0:
                sb.Append(Environment.NewLine)
                sb.Append("UserAttributes:" + Environment.NewLine)
                for userAttribute in userAtributes:
                    sb.Append(" " + userAttribute.Key.ToString() + " = " + userAttribute.Value.ToString() + Environment.NewLine)

        self.textBoxEntityInfo.Text = sb.ToString()