﻿"""
Macro to add speech support for commands
"""
import clr
import wpf
from System.Windows import Window, WindowStyle as WS, Application, SystemParameters
from System.Windows.Interop import Imaging, WindowInteropHelper
from System.Windows.Controls import ContextMenu, MenuItem
from System.IO import StreamReader, File, FileMode, FileStream, FileAccess, FileShare
from System.Runtime.Serialization.Formatters.Binary import BinaryFormatter
from System import Guid, Diagnostics
from System.Windows.Input import Key
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import DialogResult, Control, Keys
clr.AddReference("System.Speech")
from System.Speech.Recognition import SpeechRecognitionEngine, Grammar, GrammarBuilder, Choices, RecognizeMode
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import SendKeys
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import TrimbleOffice
clr.AddReference("Trimble.Vce.UI.Controls")
from Trimble.Vce.UI.Controls import VceMessageBox

cmdVersion = 0.54
def Setup(cmdData, macroFileFolder):
    cmdData.Key = "VoiceCommand"
    cmdData.EnableNoProject = True
    cmdData.HasStartup = True
    cmdData.Version = cmdVersion
    try:
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2019/12/02/tml-voice-commands"
        cmdData.HelpFile=cmdData.MacroInfo

        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "General"
        cmdData.ShortCaption = "Voice Command"
        cmdData.ToolTipTitle = "Voice Command"
        cmdData.ToolTipTextFormatted = "Execute a command using voice input"
    except:
        pass

def CreateWindow(macroFileFolder, autoStart, confidence, voiceEnabled, windowStyle):
    form = VoiceCommand(macroFileFolder, autoStart, confidence, voiceEnabled, cmdVersion)
    form.Topmost = False
    #form.ShowInTaskbar=False # set to False if you don't want to show in
    #TaskBar
    form.WindowStyle = windowStyle
    # since TBC is a WinForm app and we are using WPF, we need to do a
    # few tricks to get the window parented correctly.
    interop = WindowInteropHelper(form)
    interop.EnsureHandle() # Make sure we have a handle (creates one if its missing)
    interop.Owner = TrimbleOffice.TheOffice.MainWindow.Form.Handle #make TBC our parent
    return form

def StartupExecute(macroFileFolder):
    data = ReadVoiceStatus(macroFileFolder)
    if data and data[5]: # start if autostart turned on
        form = CreateWindow(macroFileFolder, True, data[6], data[7], data[4])
        SetWindowLocation(form, data[0],data[1],data[2],data[3])
        form.Show()

def Execute(cmd, currentProject, macroFileFolder, parameters):
    # check if we are already running
    if Application.Current:
        windows = Application.Current.Windows
        for w in windows:
            if isinstance(w, VoiceCommand):
                VceMessageBox.Show("VoiceCommand already open.", "VoiceCommand")
                return
    data = ReadVoiceStatus(macroFileFolder) # this will be an empty list first time
    if data:
        form = CreateWindow(macroFileFolder, data[5], data[6], data[7], data[4])
        SetWindowLocation(form, data[0],data[1],data[2],data[3])
    else:
        form = CreateWindow(macroFileFolder, False, 0.80, True, WS.ToolWindow)
    form.Show()

class VoiceCommand(Window): # this inherits from the WPF window control
    def __init__(self, macroFileFolder, autoStart, confidence, voiceEnabled, cmdVersion):
        with StreamReader(macroFileFolder + r"\VoiceCommand.xaml") as s:
            wpf.LoadComponent(self, s) 

        self.macroName = "Voice Command"
        self.tmlFileFolder = macroFileFolder

        self.Changed = False
        self.autoStartCtl.IsChecked = autoStart
        self.sliderCtl.Value = int(confidence * 100)
        self.mySpeechEngine = SpeechRecognitionEngine()
        self.mySpeechEngine.SpeechRecognized += self.recognizer_SpeechRecognized
        self.mySpeechEngine.SpeechRecognitionRejected += self.recognizer_SpeechRecognitionRejected
        choices = Choices()
        choices.Add("cancel")
        choices.Add("ok")
        choices.Add("tab")
        choices.Add("tab 2")
        choices.Add("tab 3")
        choices.Add("tab 4")
        choices.Add("tab 5")
        choices.Add("shift tab")
        choices.Add("shift tab 2")
        choices.Add("shift tab 3")
        choices.Add("shift tab 4")
        choices.Add("shift tab 5")

        cmdProcessor = TrimbleOffice.TheOffice.CommandProcessor
        cmdList = cmdProcessor.CommandList.Values
        for cmd in cmdList:
            if cmd.Hidden:
                continue
            if cmd.Caption:
                c = cmd.CleanCaption()
                if c:
                    choices.Add(c)

        self.mySpeechEngine.SetInputToDefaultAudioDevice()
        gb = GrammarBuilder()
        gb.Append(choices)
        self.mySpeechEngine.LoadGrammar(Grammar(gb))
        self.enableVoiceCtl.IsChecked = voiceEnabled
        #self.mySpeechEngine.RecognizeAsync(RecognizeMode.Multiple)
        self.Closing += self.ToolbarClosing
        self.SizeChanged += self.ToolbarChanged
        self.LocationChanged += self.ToolbarChanged        

        contextmenu = ContextMenu()
        self.ContextMenu = contextmenu
        mi = MenuItem()
        mi.Header = "Toggle Border Style"
        mi.Click+=self.ContextMenu_ToggleBorder
        contextmenu.Items.Add(mi)
        mi = MenuItem()
        mi.Header = "Save Layout"
        mi.Click+=self.ContextMenu_SaveLayout
        contextmenu.Items.Add(mi)
        mi = MenuItem()
        mi.Header = "Close Window"
        mi.Click+=self.ContexMenu_CloseWindow
        contextmenu.Items.Add(mi)
        self.Changed = False

    def recognizer_SpeechRecognized(self, sender, e):
        cmdCaption = e.Result.Text
        confidence = e.Result.Confidence
        t = "" #cmdCaption +  ' [{0:.2f}]'.format(confidence) 
        for alt in e.Result.Alternates:
            t = t + alt.Text +  ' [{0:.2f}]'.format(alt.Confidence) + "\n"
        self.reportCtl.Text = t
        if confidence < self.sliderCtl.Value/100.0:
            return
        if not cmdCaption:
            return
        if cmdCaption == "cancel":
            SendKeys.Send("{ESC}")
            return
        if cmdCaption == "ok":
            SendKeys.Send("{ENTER}")
            return
        if cmdCaption == "tab":
            SendKeys.Send("{TAB}")
            return
        if cmdCaption == "tab 2":
            SendKeys.Send("{TAB}{TAB}")
            return
        if cmdCaption == "tab 3":
            SendKeys.Send("{TAB}{TAB}{TAB}")
            return
        if cmdCaption == "tab 4":
            SendKeys.Send("{TAB}{TAB}{TAB}{TAB}")
            return
        if cmdCaption == "tab 5":
            SendKeys.Send("{TAB}{TAB}{TAB}{TAB}{TAB}")
            return

        if cmdCaption == "shift tab":
            SendKeys.Send("+{TAB}")
            return
        if cmdCaption == "shift tab 2":
            SendKeys.Send("+{TAB}+{TAB}")
            return
        if cmdCaption == "shift tab 3":
            SendKeys.Send("+{TAB}+{TAB}+{TAB}")
            return
        if cmdCaption == "shift tab 4":
            SendKeys.Send("+{TAB}+{TAB}+{TAB}+{TAB}")
            return
        if cmdCaption == "shift tab 5":
            SendKeys.Send("+{TAB}+{TAB}+{TAB}+{TAB}+{TAB}")
            return

        cmdProcessor = TrimbleOffice.TheOffice.CommandProcessor
        cmdList = cmdProcessor.CommandList.Values
        for cmd in cmdList:
            if cmd.Hidden:
                continue
            if not cmd.Caption:
                continue
            s = str(cmd.CleanCaption())
            if (s == cmdCaption and s):
                cmdProcessor.ExecuteCommand(cmd.Key)
                break

    def recognizer_SpeechRecognitionRejected(self, sender, e):
        cmdCaption = e.Result.Text
        t = "Rejected: [" + cmdCaption + "]"
        #confidence = e.Result.Confidence
        #t = cmdCaption +  ' [{0:.2f}]'.format(confidence) 
        #for alt in e.Result.Alternates:
        #    t = t + "\n" + alt.Text +  ' [{0:.2f}]'.format(alt.Confidence) 
        self.reportCtl.Text = t

    def ContextMenu_ToggleBorder(self, sender, e):
        if self.WindowStyle == WS['None']:
            self.WindowStyle = WS.ToolWindow
        else:
            self.WindowStyle = WS['None']

    def ContexMenu_CloseWindow(self, sender, e):
        self.Close()

    def ContextMenu_SaveLayout(self, sender, e):
        if self.Changed:
            WriteVoiceStatus(self)
        self.Changed = False

    def ToolbarChanged(self, sender, e):
        self.Changed = True

    
    def sliderCtl_ValueChanged(self, sender, e):
        self.sliderCtl.ToolTip =  '{0:.2f}'.format(self.sliderCtl.Value/100.0) 
        self.Changed = True

    def EnabledChanged(self, sender, e):
        if self.enableVoiceCtl.IsChecked :
            self.mySpeechEngine.RecognizeAsync(RecognizeMode.Multiple)
        else:
            self.mySpeechEngine.RecognizeAsyncStop()

    def ToolbarClosing(self, sender, e):
        # this is NOT called when the parent window (TBC) closes (and then this
        # window closes)
        if self.Changed:
            WriteVoiceStatus(self)

    def Window_PreviewKeyDown(self, sender, e):
        if e.Key == Key.F1:
            e.Handled = True
            Diagnostics.Process.Start(
                r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2019/12/02/tml-voice-commands")
    def Window_PreviewKeyUp(self, sender, e):
        if e.Key == Key.F1:
            e.Handled = True

def ReadVoiceStatus(macroFileFolder):
    file = macroFileFolder + r"\VoiceCommand.bin"
    try:
        if not File.Exists(file):
            return None
        
        formatter = BinaryFormatter()
        with  FileStream(file, FileMode.Open,
                FileAccess.Read, FileShare['None']) as stream:
            height = formatter.Deserialize(stream)
            width = formatter.Deserialize(stream)
            left = formatter.Deserialize(stream)
            top = formatter.Deserialize(stream)
            winStyle = formatter.Deserialize(stream)
            autoStart = formatter.Deserialize(stream)
            confidence = formatter.Deserialize(stream)
            voiceEnabled = formatter.Deserialize(stream)
            stream.Close()
 
        return (height,width,left,top,winStyle,autoStart,confidence,voiceEnabled)

    except:
        File.Delete(file)
        return None

def WriteVoiceStatus(form):
    file = MacroFileFolder + r"\VoiceCommand.bin"
    try:
        if File.Exists(file):
            File.Delete(file)

        formatter = BinaryFormatter()
        with  FileStream(file, FileMode.OpenOrCreate,
                FileAccess.Write, FileShare['None']) as stream:
            formatter.Serialize(stream, form.Height)
            formatter.Serialize(stream, form.Width)
            formatter.Serialize(stream, form.Left)
            formatter.Serialize(stream, form.Top)
            formatter.Serialize(stream, form.WindowStyle)
            formatter.Serialize(stream, form.autoStartCtl.IsChecked)
            formatter.Serialize(stream, form.sliderCtl.Value/100.0)
            formatter.Serialize(stream, form.enableVoiceCtl.IsChecked)
            stream.Close()

    except:
        File.Delete(file)

def SetWindowLocation(form, height,width,left,top):
    if height > SystemParameters.VirtualScreenHeight:
        height = SystemParameters.VirtualScreenHeight
    if width > SystemParameters.VirtualScreenWidth:
        width = SystemParameters.VirtualScreenWidth
    form.Width = width
    form.Height = height
    form.Top = top
    form.Left = left