import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System.Windows.Controls import StackPanel
from System.IO import StreamReader
from System.Windows.Input import Keyboard
from System import Type, Tuple, Array, Double
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle

from Trimble.Vce.Alignment.Linestring import Linestring,VerticalElementFactory, IVerticalNoCurve, IStationVerticalLocation

from Trimble.Vce.Geometry import Point3D, Intersections
from Trimble.Vce.Geometry.PolySeg import PolySeg

from Trimble.Vce.Interfaces import Client
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents
clr.AddReference("Trimble.Vce.UI.Controls")
try:
    from Trimble.Sdk.UI import InputMethod    
except:
    from Trimble.Sdk.Interfaces.UI import InputMethod
        
from Trimble.Vce.UI.Controls.Wpf import CoordinateEdit, DistanceEdit

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "VpiAtElev"
    cmdData.CommandName = "VpiAtElev"
    cmdData.Caption = "_Vpi At Elev"
    cmdData.UIForm = "VpiAtElev"
    cmdData.HelpFile = "Macros.chm"
    cmdData.HelpTopic = "22506"
    try:
        cmdData.Version = 1.51
        cmdData.MacroAuthor = "Trimble"
        cmdData.MacroInfo = r"https://community.trimble.com/groups/trimble-business-center-hce/blog/2018/10/22/tml-vpi-at-elevation"

        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "CAD"
        cmdData.ShortCaption = "Vpi At Elev"
        cmdData.ToolTipTitle = "Vpi At Elev"
        cmdData.ToolTipTextFormatted = "Inserts VPI at every vertical intersection of line and plane."

        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

class VpiAtElev(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\VpiAtElev.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Apply"
        buttons[1].Content = "Close"
        self.Caption = cmd.Command.Caption
        self.lineCtl.AddTypeFilter(clr.GetClrType(Linestring))

    def elevCtl_ValueChanged(self, ctrl, e):
        if e.Cause == InputMethod.Mouse:      
            self.OkClicked(None, None)

    def CancelClicked(self, thisCmd, args):
        thisCmd.CloseUICommand()

    def OkClicked(self, thisCmd, e):
        self.textBoxCtl.Text= ""
        line = self.lineCtl.Entity
        if not line:
            self.textBoxCtl.Text= "No line selected"
            return
        elev = self.elevCtl.Elevation
        if Double.IsNaN(elev):
            self.textBoxCtl.Text= "No elevation defined"
            return
        self.elevCtl.StatusMessage = ""
        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)

        try:
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                poly=line.ComputeVerticalPolySeg()
                if not poly or poly.NumberOfNodes < 2:
                    self.textBoxCtl.Text= "Line not defined"
                    return
                begSta = poly.FirstNode.Point.X
                endSta = poly.LastNode.Point.X
                acrossPoly = PolySeg()
                acrossPoly.Add(Array[Point3D]([Point3D(begSta, elev), Point3D(endSta, elev)]))
                ints = Intersections()
                nFound = 0
                if poly.Intersect(acrossPoly, True, ints):
                    for i in ints:
                        e = VerticalElementFactory.Create(clr.GetClrType(IVerticalNoCurve), clr.GetClrType(IStationVerticalLocation))
                        e.StationDistance = i.Point.X
                        e.Elevation = elev
                        line.AddVerticalElement(e)
                        nFound += 1

                self.textBoxCtl.Text= nFound.ToString() + " Elevations defined"
                # you MUST commit the change to DB or they will all be un-done (roll-back)
                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

        if (self.elevCtl.AutoTabExtSkip):
            Keyboard.Focus(self.lineCtl)
        else:
            Keyboard.Focus(self.elevCtl)
        return